package fsmv;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import be.ac.info.fundp.TVLParser.TVLParser;

import fsmv.phi.Configuration;
import fsmv.predicate.Predicate;

public class FSMvRho {
	
	private final List<Predicate> conjuncts;
	
        
        //TODO Here we have to capture the feature diagram competely from the tvl file.
        //Create a varible for each feature.
	public FSMvRho(File tvlFile, boolean design)
	{
            this.conjuncts=new ArrayList<>();

            try
            {
                TVLParser tvlParser=new TVLParser(tvlFile);
                String equation=tvlParser.getBooleanForm();

                //System.out.println("FSMvRho.new(): File="+tvlFile.getName()+"\npredicateInTVL:"+equation);

                String[] equArray=equation.split("\n");
                for(int i=0;i<equArray.length;i++)
                {
                    equArray[i]=equArray[i].trim();
                    if(equArray[i].startsWith("(") && equArray[i].endsWith(");") &&
                       (
                            equArray[i].contains("&&")
                            ||
                            equArray[i].contains("||")
                            ||
                            equArray[i].contains("!")
                       ))
                    {
                        StringBuilder result=new StringBuilder("");

                        equArray[i]=equArray[i].substring(0, equArray[i].length()-1);

                        String[] conjuncts=equArray[i].split("&&");
                        for(int j=0;j<conjuncts.length;j++)
                        {
                            if(j<conjuncts.length-1)
                                result.append("and(");

                            conjuncts[j]=conjuncts[j].trim();
                            while(conjuncts[j].startsWith("(") && conjuncts[j].endsWith(")"))
                            {
                                conjuncts[j]=conjuncts[j].substring(1, conjuncts[j].length()-1);
                                conjuncts[j]=conjuncts[j].trim();
                            }
                            //System.out.println(conjuncts[j]);
                            String[] variables=conjuncts[j].split("\\|\\|");
                            for(int k=0;k<variables.length;k++)
                            {
                                //System.out.println(variables[k]);
                                if(k<variables.length-1)
                                    result.append("or(");


                                variables[k]=variables[k].trim();
                                if(variables[k].startsWith("!"))
                                {
                                    variables[k]=variables[k].substring(1);
                                    variables[k]=variables[k].trim();
                                    variables[k]="not("+variables[k]+")";
                                }

                                variables[k]=variables[k].replace(".", "."+(design?"D":"R")+".");

                                int index=variables[k].lastIndexOf("_");
                                if(index>0 && index <variables[k].length())
                                {
                                    StringBuilder sb=new StringBuilder(variables[k]);
                                    sb.setCharAt(index, '=');
                                    variables[k]=sb.toString();
                                }
                                result.append(variables[k]);

                                if(k<variables.length-1)
                                    result.append(",");
                                else
                                {
                                    for(int x=0;x<variables.length-1;x++)
                                        result.append(")");
                                }

                            }
                            if(j<conjuncts.length-1)
                                result.append(", ");
                            else
                            {
                                for(int x=0;x<conjuncts.length-1;x++)
                                    result.append(")");
                            }
                        }

                       //System.out.println("FSMvRho.new(): predicateFromTVL:"+result.toString()); 
                        this.conjuncts.add(new Predicate(result.toString()));
                        //System.out.println(result.toString());
                    }
                }
            }catch(IOException e)
            {
                e.printStackTrace();
                System.out.println("The TVL file "+tvlFile+" cannot be parsed");
            }
	}
	
	public FSMvRho(List<Predicate> _conjunct)
	{
		conjuncts=_conjunct;
	}
        
        @Override
	public String toString()
	{
		String text;
		text="<rho>\n";
		for(int i=0;i<conjuncts.size();i++)
			text+=" <conjunct>"+conjuncts.get(i)+"</conjunct>\n";
		text+="</rho>\n";
		return text;
	}
	public List<Predicate> getConjuncts() {
		return conjuncts;
	}

	public static FSMvRho merge(List<FSMvRho> rhos)
	{
		if(rhos.size()==0)
			return new FSMvRho(new ArrayList<Predicate>());
		if(rhos.size()==1)
			return rhos.get(0);
		
		FSMvRho rho1=rhos.remove(0);
		FSMvRho rho2=rhos.remove(0);
		
		FSMvRho mergeRho=new FSMvRho(new ArrayList<Predicate>());
		
		mergeRho.conjuncts.addAll(rho1.conjuncts);
		mergeRho.conjuncts.addAll(rho2.conjuncts);
		
		rhos.add(0, mergeRho);
		return FSMvRho.merge(rhos);
	}
	
	
	public boolean isSatisfiedBy(Configuration conf)
	{
		for(Predicate p:this.conjuncts)
		{
			if(!p.isSatisfiedBy(conf))
				return false;
		}
		return true;
	}
	
	
	
}
