package fsmvVerifier;

import java.io.*;
import java.util.*;

import fsmv.*;
import fsmv.constants.IFSMVConstants;
import fsmv.phi.Configuration;
import fsmv.phi.Phi;
import fsmv.phi.PhiCouple;
import fsmv.predicate.Node;

import fsmvVerifier.generator.FSMvRhoGenerator;
import fsmvVerifier.generator.PromelaGenerator;
import fsmvVerifier.generator.QBFGenerator;
import fsmvVerifier.generator.SingleVariantPromelaGenerator;
import java.util.Map.Entry;
import toools.extern.Proces;
import toools.extern.ProcessOutput;
import toools.io.file.RegularFile;

public class Verifier {

    final String selectedDirPath;
    
    Map<FSMv, FSMv> listOfFeature;
    final Set<Phi> listOfPhi;
    final FSMvRho rhod;
    final FSMvRho rhor;
    final Map<FSMv, String> executionTime;
    
    public Verifier(String selectedDirPath, File file) throws IOException 
    {
        //TODO remove selec.. from the param but get it from file.
    	this.selectedDirPath=selectedDirPath;
        
        this.executionTime=new HashMap<>();
        
        this.setListOfFeatureFromPropertyFile(file);
        
        FSMvRhoGenerator gen=new FSMvRhoGenerator(selectedDirPath);
        
        //System.out.println("Verifier:constructor()");
        
        this.rhod = gen.createCompositionalConstraints(true);
        this.rhor = gen.createCompositionalConstraints(false);
        
        
        this.listOfPhi = new HashSet<>();
    }

    /**
     * Reads a "properties" file, and returns it as a Map
     * (a collection of key/value pairs).
     *
     * @param filename  The properties filename to read.
     * @param delimiter The string (or character) that separates the key
     *                  from the value in the properties file.
     * @return The Map that contains the key/value pairs.
     * @throws Exception
     */
    private void setListOfFeatureFromPropertyFile(File file)
            throws IOException {
        
            listOfFeature=new HashMap<>();
            addFeaturePairsFromFile(file);
    }
    
    public void addFeaturePairsFromFile(File file) throws IOException 
    {
        try (BufferedReader reader = new BufferedReader(new FileReader(file))) 
        {
            String line;
            while ((line = reader.readLine()) != null ) {
                
                line=line.trim();
                if (line.length() < 3) 
                {
                    continue;
                }
                if (line.charAt(0) == '#') 
                {
                    continue;
                }
                if(!line.contains("="))
                {
                    continue;
                }
                if(!addFeaturePairFromTextLine(line))
                    break;
            }
        }
    }

    public boolean addFeaturePairFromTextLine(String line) 
    {
        //System.out.println(line);
        int delimPosition = line.indexOf("=");
        String key = line.substring(0, delimPosition).trim();
        String value = line.substring(delimPosition + 1).trim();

        FSMv req_machine=null, des_machine=null;

        if(!key.contains(":"))
        {
            req_machine = new SingleFSMv(selectedDirPath + IFSMVConstants.FSMR_FILE_PATH + key);
        }
        else
        {
            req_machine = new ComposedFSMv(selectedDirPath + IFSMVConstants.FSMR_FILE_PATH, key);
        }

        if(!value.contains(":"))
        {
            des_machine = new SingleFSMv(selectedDirPath + IFSMVConstants.FSMD_FILE_PATH + value);
        }
        else
        {
            des_machine = new ComposedFSMv(selectedDirPath + IFSMVConstants.FSMD_FILE_PATH, value);
        }
        
        if(req_machine==null || des_machine==null)
            return false;
        
        for(Entry<FSMv,FSMv> entry: this.listOfFeature.entrySet())
        {
            if(entry.getKey().getName().equals(des_machine.getName()) && entry.getValue().getName().equals(req_machine.getName()))
            {
                //System.out.println("Feature "+des_machine.getName()+" already present.");
                return true;
            }
        }
        this.listOfFeature.put(des_machine, req_machine);
        this.executionTime.put(des_machine, "undefined");
        return true;
    }

    public Map<FSMv, FSMv> getListOfFeature() {
        return listOfFeature;
    }

    public Set<Phi> getListOfPhi() {
        return listOfPhi;
    }

    public FSMvRho getRhod() {
        return rhod;
    }
    
    public FSMvRho getRhor() {
        return rhor;
    }
    
    public String getSelectedDirPath()
    {
        return this.selectedDirPath;
    }

    public Map<FSMv, String> getExecutionTime() 
    {
            return executionTime;
    }
    
    
    /**
     * This method checks the conformance of the FSMd fsmd to its requirement.
     * @param fsmd the checked FSMd
     * @return  the Phi mapping. Note that this mapping is also added to the list of Phi mapping to avoid double checking.
     */
    public Phi RefinementChecking(FSMv fsmd) 
    {
        //Build a list of all the pair
        List<Configuration[]> VAConf = new ArrayList<>();
        for (int i = 0; i < fsmd.getListOfValidConfiguration().size(); i++) 
        {
            for (int j = 0; j < listOfFeature.get(fsmd).getListOfValidConfiguration().size(); j++) 
            {
                Configuration[] c = new Configuration[2];
                c[0] = fsmd.getListOfValidConfiguration().get(i);
                c[1] = listOfFeature.get(fsmd).getListOfValidConfiguration().get(j);
                VAConf.add(c);
            }
        }
        //System.out.println("Feature Name: "+fsmd.getName());
        //System.out.println("Initial size of varConf:"+VAConf.size()+"\n");
        
        
        try {
            if(! new File(selectedDirPath+"/tmp").exists())
                Runtime.getRuntime().exec("mkdir "+selectedDirPath+"/tmp").waitFor();

            this.makePromelafile(selectedDirPath+"/tmp/" + fsmd.getName()+".pml", fsmd);
            
            Runtime.getRuntime().exec("spin -a "+selectedDirPath+"/tmp/" + fsmd.getName()+".pml").waitFor();
            
            Runtime.getRuntime().exec("mv ./pan.b "+selectedDirPath+"/tmp/pan.b").waitFor();
            Runtime.getRuntime().exec("mv ./pan.c "+selectedDirPath+"/tmp/pan.c").waitFor();
            Runtime.getRuntime().exec("mv ./pan.h "+selectedDirPath+"/tmp/pan.h").waitFor();
            Runtime.getRuntime().exec("mv ./pan.m "+selectedDirPath+"/tmp/pan.m").waitFor();
            Runtime.getRuntime().exec("mv ./pan.p "+selectedDirPath+"/tmp/pan.p").waitFor();
            Runtime.getRuntime().exec("mv ./pan.t "+selectedDirPath+"/tmp/pan.t").waitFor();
            
            Process p2 = Runtime.getRuntime().exec("gcc "+selectedDirPath+"/tmp/pan.c -DPRINTF -o "+selectedDirPath+"/tmp/pan");
            p2.waitFor();
            //System.out.println("gcc done");
            Process p3 = Runtime.getRuntime().exec(selectedDirPath+"/tmp/pan -X -m10000000 -w25 -c0 -a -n");
            BufferedReader br = new BufferedReader(new InputStreamReader(p3.getInputStream()));
            String read_line = null;
            List<String> output = new ArrayList<>();
            while ((read_line = br.readLine()) != null) {
                //System.out.println(line);
                if (read_line.startsWith("pan: elapsed time")) {
                    String panTimeStr = (String) read_line.subSequence(18, read_line.length());
                    //System.out.println("panTimeStr:"+panTimeStr);
                    String[] str = panTimeStr.split(" ");
                    executionTime.put(fsmd, str[0]);
                    }
                if(read_line.startsWith("<FSMv>")&& read_line.endsWith(";"))
                {
                    if(output.isEmpty() || !output.get(output.size()-1).equals(read_line.substring(6)))
                        output.add(read_line.substring(6));
                }
                    
            }
            p3.waitFor();
            
            //System.out.println(output.size());
            
            //Each line corresponds to a no conforming pair of configurations
            for(String line: output)
            {
                if(line.equals(""))
                    continue;
                
                Configuration c_des = new Configuration();
                Configuration c_req = new Configuration();
                
                String[] var_values=line.split(";");
                
                //Each var_value has the form: featureName_[R|D]_variableName=[indexOfValue]
                for(String var_value: var_values)
                {
                    String[] vv=var_value.split(":");
                    String variableName=vv[0];//.replaceAll("_", ".");//.substring(fsmd.getName().length()+1);
                    
                    
                    boolean design=variableName.contains("_D_");
                    FSMv          fsmv       =design ? fsmd  : listOfFeature.get(fsmd);
                    Configuration currentConf=design ? c_des : c_req;
                    
                    variableName=variableName.replace("_"+fsmv.getType()+"_", "."+fsmv.getType()+".");
                    
                    //System.out.println("variableName="+variableName);
                    
                    FSMvVariable variable=null;
                    
                    //Find the FSMvVariable object from its name
                    for(FSMvVariable var:fsmv.getVariables())
                    {
                        if(var.getQualifiedName().equals(variableName))
                        {
                            variable=var;
                            break;
                        }
                    }
                    //System.out.println("variable="+variable+", vv[1]="+vv[1]);
                    
                    //Get the value as a string from its index
                    String value=variable.getValues().get(new Integer(vv[1].trim()).intValue());
                    
                    currentConf.getListOfVariable_value().put(variableName, value);
                }
                
                //Remove the computed conf from the list of valid conf
                for (int j = 0; j < VAConf.size(); j++) {
                    //System.out.println(VAConf.get(j)[0].equals(c_des)+"  "+VAConf.get(j)[1].equals(c_req));
                    if (VAConf.get(j)[0].equals(c_des) && VAConf.get(j)[1].equals(c_req)) 
                    {
                        //System.out.println("I remove this pair of conf"+ c_des + c_req);
                        VAConf.remove(j);
                        break;
                    }
                }
            }
            
            p3.getInputStream().close();
            //br.close();
            
            Runtime.getRuntime().exec("rm -f "+selectedDirPath+"/tmp/pan.b").waitFor();
            Runtime.getRuntime().exec("rm -f "+selectedDirPath+"/tmp/pan.c").waitFor();
            Runtime.getRuntime().exec("rm -f "+selectedDirPath+"/tmp/pan.h").waitFor();
            Runtime.getRuntime().exec("rm -f "+selectedDirPath+"/tmp/pan.m").waitFor();
            Runtime.getRuntime().exec("rm -f "+selectedDirPath+"/tmp/pan.p").waitFor();
            Runtime.getRuntime().exec("rm -f "+selectedDirPath+"/tmp/pan.t").waitFor();
            Runtime.getRuntime().exec("rm -f "+selectedDirPath+"/tmp/pan.pre").waitFor();
            Runtime.getRuntime().exec("rm -f "+selectedDirPath+"/tmp/pan").waitFor();
            
            //Runtime.getRuntime().exec("rm -f "+selectedDirPath+"/tmp/"+ fsmd.getName()+".pml").waitFor();;
            
        } catch (IOException | InterruptedException | NumberFormatException e) 
        {
            e.printStackTrace();
        }

        //System.out.println("Size of varConf:"+VAConf.size()+"\n");

        //Verification of the presence of every design configuration in VAConf
        Phi phi = new Phi(fsmd, listOfFeature.get(fsmd));
        HashSet<Configuration> illegalConfig = new HashSet<>();
        for (int i = 0; i < fsmd.getListOfValidConfiguration().size(); i++) 
        {
            boolean found = false;
            for (int j = 0; j < VAConf.size(); j++) {
                if (fsmd.getListOfValidConfiguration().get(i).equals(VAConf.get(j)[0])) 
                {
                    found = true;
                    break;
                }
            }
            if (!found) 
            {
                //System.out.println("Illegal configuration 100:" + fsmd.getListOfValidConfiguration().get(i));
                illegalConfig.add(fsmd.getListOfValidConfiguration().get(i));
            }
        }
        //Translation of VAConf in phi
        List<PhiCouple> phiList = new ArrayList<>();
        for (int i = 0; i < VAConf.size(); i++) {
            phiList.add(new PhiCouple(VAConf.get(i)[0], VAConf.get(i)[1]));
        }
        phi.setListOfCorrespondanceMapping(phiList);
        phi.setListOfIllegalConfigurations(illegalConfig);
        //setVaribales

        //System.out.println("fsmd.getVariables(): "+fsmd.getVariables());
        //System.out.println("this.listOfFeature.get(fsmd).getVariables(): "+this.listOfFeature.get(fsmd).getVariables());

        //phi.setFsmvDesignVariable(fsmd.getVariables());
        //phi.setFsmvReqVariable(this.listOfFeature.get(fsmd).getVariables());
        //phi.setfSMvDesignRho(fsmd.getRho());
        //phi.setfSMvReqRho(this.listOfFeature.get(fsmd).getRho());
        this.listOfPhi.add(phi);
        return phi;
    }

    private boolean makePromelafile(String fileName, FSMv fsmd) 
    {
        try 
        {
            FileWriter file = new FileWriter(fileName);
            file.write(new PromelaGenerator(fsmd, this.listOfFeature.get(fsmd)).exportInSpin());
            file.close();
            return true;
        } catch (IOException e) {
            //System.out.println(e);
            return false;
        }
    }

    public void FullRefinementChecking() 
    {
        List<FSMv> fsmds = new ArrayList<>(this.listOfFeature.keySet());
        for (FSMv fsmd: fsmds) 
        {
            if(!this.listOfPhiContains(fsmd))
            {
                System.out.println("Start of the FSMd "+ fsmd.getName());
                RefinementChecking(fsmd);
                System.out.println(fsmd.getName() + " done!\n");
            }
        }
    }
    
    private boolean listOfPhiContains(FSMv fsmd)
    {
        for(Phi phi: this.listOfPhi)
            if(phi.getDesign().equals(fsmd))
                return true;
        return false;
    }
    
    public String[] splLevelChecking() 
    {
        QBFGenerator qbfGenerator=new QBFGenerator(selectedDirPath, this.getListOfPhi(), this.rhod, this.rhor);
        qbfGenerator.generateQBFFile();
        
        return runCirQit(qbfGenerator.getQBFDesVariables(), qbfGenerator.getQBFReqVariables());
    }
    
    /**
     * To run the CirQit solver on the generated temp.qbf file. 
     */
    private String[] runCirQit(Map<String, String> qbfDesVariables, Map<String, String> qbfReqVariables) 
    {
        //for(Entry<String,String> entry: qbfReqVariables.entrySet())
        //    System.out.println(entry.getKey()+"<=>"+entry.getValue());
        
    	 try {
             byte[] in=new RegularFile(selectedDirPath+"/tmp/temp.qbf").getContent();
             ProcessOutput po=Proces.rawExec("boole2qpro", in);
             byte[] output=new byte[0];
             output=po.getStdout();
             
             RegularFile tempQpro=new RegularFile(selectedDirPath+"/tmp/temp.qpro");
             tempQpro.setContent(output);
             
             output=new byte[0];
             po=Proces.rawExec("CirQit3.1.7", "-qpro", "1", "-proof", selectedDirPath+"/tmp/problem.txt", selectedDirPath+"/tmp/proof.txt", selectedDirPath+"/tmp/temp.qpro");
             output=po.getStderr();
             String output_s = new String(output);
             
             boolean illegalConfDetected=false;
             
             if(output_s.contains("no solution found"))
             {
                 illegalConfDetected=true;
             }
             else
             {
                 Proces.exec("rm", selectedDirPath+"/tmp/proof.txt", selectedDirPath+"/tmp/problem.txt");
             }
             
             
             return buildReturnValue(illegalConfDetected, qbfDesVariables, qbfReqVariables);
            
         } catch (IOException e) 
         {
             e.printStackTrace();
             String[] ret={"Error occured."};
             return ret;
         }
	}

    private String[] buildReturnValue(boolean illegalConfDetected, Map<String, String> qbfDesVariables, Map<String, String> qbfReqVariables) throws IOException 
    {
        if(!illegalConfDetected)
         {
             String[] ret={"There is no invalid configuration.\nThe design refines the requirement.\n"};
             return ret;
         }
        
         byte[] in=new RegularFile(selectedDirPath+"/tmp/proof.txt").getContent();
         Proces.exec("rm", "-f", selectedDirPath+"/tmp/proof.txt", selectedDirPath+"/tmp/problem.txt");

         ProcessOutput po=Proces.rawExec("topU.py", in);
         byte[] output=po.getStdout();
         String output_s=new String(output);
         output_s=output_s.substring(23);
         output_s=output_s.trim();
         String[] varList=output_s.split(" ");
         for(int i=0;i<varList.length;i++)
         {
             //System.out.println(varList[i]);
             int index=-1;

             try{
             index=new Integer(varList[i]).intValue();
             }catch(NumberFormatException e){index=-1;}

             if(index<0)
             {
                 varList[i]="";
                 continue;
             }

             index--;
             boolean design=(index<=qbfDesVariables.size());
             varList[i]=(design?"d":"r")+(design?""+index:""+(index-qbfDesVariables.size()));

             Map<String,String> varMap=(design)?qbfDesVariables:qbfReqVariables;

            for(Map.Entry<String, String> entry : varMap.entrySet()) 
             {
                 if(entry.getValue().equals(varList[i]))
                 {
                    varList[i]=entry.getKey();
                    break;
                 }
             }

            int dotIndex=varList[i].lastIndexOf(".");
             StringBuilder sb=new StringBuilder(varList[i]);
             sb.setCharAt(dotIndex,'=');
             varList[i]=sb.toString();
         }
         return varList;
    }
    
    /**
     * To convert the expression text written on the requirement variables in the native format
     * to its design equivalent with either the native of the Promela format.
     * @param expression The expression to convert
     * @param promela true to set the output format to Promela or false to set the output format to natif
     * @return the converted expression
     */
    public String convertExpression(String expression, boolean promela) 
    {
        try{
            Node tree=Node.buildTree(expression.trim());
            
            //System.out.println(""+tree.getContent()+", Left="+tree.getLeftSon().getContent()+", Right="+tree.getRightSon().getContent());
            
            if(!exploreTree(tree, promela))
                throw new RuntimeException("Check features, variables, and values names");
            
            //System.out.println(""+tree.getContent()+", Left="+tree.getLeftSon().getContent()+", Right="+tree.getRightSon().getContent());
            
            if(promela)
                return tree.toPromelaString();
            else return tree.toString();
            
        }catch(RuntimeException e)
        {
            e.printStackTrace();
            return "Error while parsing the expression:\n"
            	+e.getClass().getCanonicalName();
        }
    }
    
    private boolean exploreTree(Node node, boolean promela)
    {
        if(node.getLeftSon()!=null)
            exploreTree(node.getLeftSon(), promela);
        if(node.getRightSon()!=null)
            exploreTree(node.getRightSon(), promela);
        if(node.getLeftSon()==null && node.getRightSon()==null)
            if(!convertVarValue(node, promela))
                return false;
        
        return true;
                   
    }

    private boolean convertVarValue(Node node, boolean promela) 
    {
        //System.out.println(""+node.getContent());
        
        String vv=node.getContent();
        
        vv=vv.trim();
        
        if("".equals(vv))
            return false;
        
        String[] v_v=vv.split("=");
        String[] variable=v_v[0].split("\\.");
        String value=v_v[1].trim();
        
        FSMv fsmr=getFeatureFromNameAndType(variable[0], variable[1].trim().charAt(0));
        if(fsmr.getType()!='R')
            return false;
        
        System.out.println("Feature found");
        
       FSMvVariable var =fsmr.getVariableFromName(variable[2]);
        
       if(var==null)
           return false;
       
        System.out.println("Variable found"+var);
       
        if(!var.contains(value))
            return false;
        
        System.out.println("Value found");
        
        //all is good :)
        
        //Find the configurations where the variable var has the value val
        List<Configuration> listReqConf=new ArrayList<>();
        
        for(Configuration conf:fsmr.getListOfValidConfiguration())
        {
            for(Map.Entry<String,String> entry:conf.getListOfVariable_value().entrySet())
            {
                if(entry.getKey().equals(var.getQualifiedName()) && entry.getValue().equals(value))
                {
                    listReqConf.add(conf);
                    break;
                }
                    
            }
        }
        
        //Find the good phi
        Phi phi=null;
        for(Phi current_phi:listOfPhi)
        {
            if(current_phi.getRequirement()==fsmr)
                phi=current_phi;
        }
        
        List<Configuration> listDesConf=new ArrayList<>();
        
        for(Configuration conf: listReqConf)
        {
            for(Configuration conf_des: phi.getDesign().getListOfValidConfiguration())
            {
                if(phi.getCorrespondancesTo(conf_des).contains(conf))
                {
                    if(! listDesConf.contains(conf_des))
                      listDesConf.add(conf_des);
                }
            }
        }
        
        StringBuilder text=new StringBuilder("");
        String delim="";
        for(int i=0; i<listDesConf.size(); i++)
        {
            text.append(delim);
            
            if(i<listDesConf.size()-1)
                text.append("or(");
            
            if(promela)
                    text.append(listDesConf.get(i).toPromelaString());
            else text.append(listDesConf.get(i).toPredicateString());
            
            delim=",";
        }
        for(int i=1; i<listDesConf.size(); i++)
            text.append(")");
        
        String result=text.toString();
        
        if(promela)
        {
            result=result.substring(2, result.length()-listDesConf.size()+2);
            result=result.replaceAll(",or\\(", " || ");
            
            result=result.replaceAll(",", " || ");
        }
        
        //TODO fix it
        //result=result.replace("(", "("+fsmr.getFeatureName()+"."+fsmr.getType()+".");
        
        node.setContent(result);
        
        return true;
    }
    
    private FSMv getFeatureFromNameAndType(String name, char type)
    {
        for(FSMv fsmv:this.listOfFeature.keySet())
        {
            if(fsmv.getName().equals(name))
            {
                if(type==fsmv.getType())
                    return fsmv;
                else if(type==this.getListOfFeature().get(fsmv).getType())
                    return this.getListOfFeature().get(fsmv);
                else return null;
            }
        }
        return null;
    }
    
    
    public String generateDesign(Configuration conf, String property)
    {
        
        //Convert property in neverClaim
        String neverClaim=null;
        try {
            String commandLine="spin -f \"!("+property+")\"";
            Process p = Runtime.getRuntime().exec(commandLine);
            BufferedReader br = new BufferedReader(new InputStreamReader(p.getInputStream()));
            String read_line = null;
            StringBuilder output = new StringBuilder();
            while ((read_line = br.readLine()) != null) {
                //System.out.println(line);
                output.append(read_line);
                output.append("\n");
            }
            p.waitFor();
            neverClaim=output.toString();
        } catch (IOException|InterruptedException ex) {
            ex.printStackTrace();
        }
       
        if(neverClaim==null || !neverClaim.startsWith("never"))
            return "<Error>Syntax error in the never claim property\n"+neverClaim;
        
        //Generate
        new SingleVariantPromelaGenerator(selectedDirPath+"/tmp/fullDesign.pml",new ArrayList<>(this.getListOfFeature().keySet()), conf,neverClaim).generate();

        //run and get result
        String outputString=null;
        try {
            
            Runtime.getRuntime().exec("spin -a "+selectedDirPath+"/tmp/fullDesign.pml").waitFor();
            
            Runtime.getRuntime().exec("mv ./pan.b "+selectedDirPath+"/tmp/pan.b").waitFor();
            Runtime.getRuntime().exec("mv ./pan.c "+selectedDirPath+"/tmp/pan.c").waitFor();
            Runtime.getRuntime().exec("mv ./pan.h "+selectedDirPath+"/tmp/pan.h").waitFor();
            Runtime.getRuntime().exec("mv ./pan.m "+selectedDirPath+"/tmp/pan.m").waitFor();
            Runtime.getRuntime().exec("mv ./pan.p "+selectedDirPath+"/tmp/pan.p").waitFor();
            Runtime.getRuntime().exec("mv ./pan.t "+selectedDirPath+"/tmp/pan.t").waitFor();
            
            Process p2 = Runtime.getRuntime().exec("gcc "+selectedDirPath+"/tmp/pan.c -o "+selectedDirPath+"/tmp/pan");
            p2.waitFor();
            
            Process p3 = Runtime.getRuntime().exec(selectedDirPath+"/tmp/pan -X -m10000000 -w25 -a -n");// -c0");
            BufferedReader br = new BufferedReader(new InputStreamReader(p3.getInputStream()));
            String read_line = null;
            StringBuilder output = new StringBuilder();
            while ((read_line = br.readLine()) != null) {
                output.append(read_line);
                output.append("\n");
            }
            p3.waitFor();
            
            Runtime.getRuntime().exec("rm -f "+selectedDirPath+"/tmp/pan.b").waitFor();
            Runtime.getRuntime().exec("rm -f "+selectedDirPath+"/tmp/pan.c").waitFor();
            Runtime.getRuntime().exec("rm -f "+selectedDirPath+"/tmp/pan.h").waitFor();
            Runtime.getRuntime().exec("rm -f "+selectedDirPath+"/tmp/pan.m").waitFor();
            Runtime.getRuntime().exec("rm -f "+selectedDirPath+"/tmp/pan.p").waitFor();
            Runtime.getRuntime().exec("rm -f "+selectedDirPath+"/tmp/pan.t").waitFor();
            Runtime.getRuntime().exec("rm -f "+selectedDirPath+"/tmp/pan.pre").waitFor();
            Runtime.getRuntime().exec("rm -f "+selectedDirPath+"/tmp/pan").waitFor();
            
            
            outputString=output.toString();
             
             
        } catch (IOException|InterruptedException ex) {
            ex.printStackTrace();
        }
        
        return outputString;
    }
}
