/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package fsmvVerifier.generator;

import fsmv.ComposedFSMv;
import fsmv.FSMv;
import fsmv.SingleFSMv;
import fsmv.FSMvEvent;
import fsmv.FSMvVariable;
import fsmv.phi.Configuration;
import fsmvVerifier.generator.promela.ChannelGenerator;
import fsmvVerifier.generator.promela.ChannelSenderGenerator;
import fsmvVerifier.generator.promela.ModelGenerator;
import fsmvVerifier.generator.promela.RunStatementGenerator;
import fsmvVerifier.generator.promela.StateGenerator;
import fsmvVerifier.generator.promela.StateIsErrorGenerator;
import fsmvVerifier.generator.promela.StateVariableGenerator;
import fsmvVerifier.generator.promela.ValuesGenerator;
import fsmvVerifier.generator.promela.VariableGenerator;
import java.util.List;

/**
 *
 * @author jvmillo
 */
public class PromelaGenerator 
{
	private final FSMv fsmd;
    private final FSMv fsmr;
    private int order;
	private final int nbProcesses;
    
    public PromelaGenerator(FSMv fsmd, FSMv fsmr)
    {
        this.fsmd=fsmd;
        this.fsmr=fsmr;
        this.order=0;
        
        int tmp=1;
        if(fsmd instanceof SingleFSMv)
            tmp++;
        else if(fsmd instanceof ComposedFSMv)
            tmp+=((ComposedFSMv)fsmd).getNbOfSingleFSMv();
        if(fsmr instanceof SingleFSMv)
            tmp++;
        else if(fsmr instanceof ComposedFSMv)
            tmp+=((ComposedFSMv)fsmr).getNbOfSingleFSMv();
        
        nbProcesses=tmp;
    }

    public String exportInSpin() 
    {
        StringBuilder text = new StringBuilder("");
        try {
            
            //Define design values
            new ValuesGenerator().run(text, fsmd);
             
            //Define requirement values
            new ValuesGenerator().run(text, fsmr);
            
            //Define events
            this.generateEvent(text, fsmd.getEvents());
            
            //Define design states
            new StateGenerator().run(text, fsmd);
            
            //Define requirement states
            new StateGenerator().run(text, fsmr);
            
            //Define the channel
            //text.append("/*this channel is use to forward the events in ").append(fsmv.getName()).append("_").append(fsmv.getType()).append(" from environment*/\n");
        
            new ChannelGenerator().run(text, fsmr);
            
            new ChannelGenerator().run(text, fsmd);
            
            //Define the two state variables
            //text.append("/*State variable of ").append(fsmv.getName()).append("_").append(fsmv.getType()).append("*/\n");
            new StateVariableGenerator().run(text, fsmr);
            
            new StateVariableGenerator().run(text, fsmd);
            
            //Define the requirement variables
            new VariableGenerator().run(text, fsmr);
            
            //Define the design variables
            new VariableGenerator().run(text, fsmd);
            
            //Define the flag to statically schedule the proctype execution
            text.append("/*The flag is used to statically schedule the proctypes*/\n");
            text.append("byte flag;\n");

            text.append("\n\n");

            //The environment proctype
            this.generateEnvironmentModel(text);

            //The design proctype
            ModelGenerator mg1=new ModelGenerator(order,this.nbProcesses);
            mg1.run(text, fsmd);

            //The requirement proctype
            ModelGenerator mg2=new ModelGenerator(mg1.getOrder(),this.nbProcesses);
            mg2.run(text, fsmr);
            
            order=mg2.getOrder();
            
            //The never claim
            this.generateNeverClaim(text,fsmd.getName());
            
            //The initialization
            this.generateInitBlock(text,fsmd.getName());
            
            return text.toString();

        } catch (Exception e) {
            System.out.println(text);
            e.printStackTrace();
        }
        return null;
    }

    private void generateEvent(StringBuilder text, List<FSMvEvent> events) {
        
            
            text.append("/*The Events*/\n");
            for (int i = 0; i < events.size(); i++) {
                text.append("#define  evt_").append(events.get(i).getEvent()).append(" ").append(i).append("\n");
            }
            text.append("\n\n");
    }

    private void generateEnvironmentModel(StringBuilder text) {
        
    	List<FSMvEvent> events=fsmd.getEvents();
		text.append("proctype environmentModel(){ \n  do \n	::flag==");
        text.append(this.order);
        this.order++;
        text.append(" -> flag=");
        text.append(this.order);
        
        text.append("; atomic{if \n");
        for (int i = 0; i < events.size(); i++) 
        {
            text.append("	::(1)-> ");
            new ChannelSenderGenerator(events.get(i)).run(text, fsmd);
            text.append("\n                ");
            new ChannelSenderGenerator(events.get(i)).run(text, fsmr);
            text.append("\n");
        }
        text.append("	fi;}\n  od; \n };\n\n\n");
    }

    private void generateNeverClaim(StringBuilder text, String featureName) 
    {
    	text.append("\n\n /*never claim definition*/\n");
        text.append("never { \n TO_init: \n	if\n 	::(flag==");
        text.append(this.order);
        text.append(" && ");
        text.append("(");
        new StateIsErrorGenerator().run(text, fsmr);
        text.append(")");
        text.append(" && ");
        text.append(" (!(");
        
        new StateIsErrorGenerator().run(text, fsmd);
        
        text.append(")))");
        text.append("->");
        text.append("printf(\"<FSMv>");
        for (int i = 0; i < fsmr.getVariables().size(); i++) {
            text.append(getVariableName(fsmr.getVariables().get(i))).append(":%d;");
        }
        for (int i = 0; i < fsmd.getVariables().size(); i++) 
        {
            text.append(getVariableName(fsmd.getVariables().get(i))).append(":%d;");
        }
        text.append("\\n\"");
        for (int i = 0; i < fsmr.getVariables().size(); i++) 
        {
            text.append(",");
            text.append(getVariableName(fsmr.getVariables().get(i)));
        }
        for (int i = 0; i < fsmd.getVariables().size(); i++) 
        {
            text.append(", ");
            text.append(getVariableName(fsmd.getVariables().get(i)));
        }
        text.append(");\n");
        text.append(" goto accept_S9 \n	::(1) -> goto TO_init \n	fi;\n");
        text.append("accept_S9:\n	if\n	::(1) -> goto TO_init \n	fi; \n } \n\n\n\n");
        
    }

    private void generateInitBlock(StringBuilder text, String featureName) 
    {
    	text.append("init{ \n flag=0; \n");
        
        //Set a random requirement configuration amongst the valid configurations
        text.append(generateVariableInitBlock(fsmr));
        
        //Set a random design configuration amongst the valid configurations
        text.append(generateVariableInitBlock(fsmd));
        
        //Launch
        text.append("	atomic { \n");
        text.append("		run environmentModel(); \n");
       
       new RunStatementGenerator().run(text, fsmd);
       new RunStatementGenerator().run(text, fsmr);
       
        text.append("		}\n");
       text.append("	}");
    }
    
    private StringBuilder generateVariableInitBlock(FSMv fsmv)
    {
            StringBuilder text=new StringBuilder("");
            text.append("atomic{\n	 if \n");
            for (int i = 0; i < fsmv.getListOfValidConfiguration().size(); i++) 
            {
                Configuration config = fsmv.getListOfValidConfiguration().get(i);

                text.append("	:: (1)-> ");
                for (int j = 0; j < fsmv.getVariables().size(); j++) {
                    text.append(getVariableName(fsmv.getVariables().get(j)))
                            .append("=")
                        .append(getValueName(fsmv.getVariables().get(j), config.get(fsmv.getVariables().get(j).getQualifiedName())))
                            .append("; ");
                }
                text.append("\n");
            }
            text.append("	fi;} \n\n");
            return text;
    }
    
    private String getVariableName(FSMvVariable var)
    {
        return var.getQualifiedName().replace(".", "_");
    }
    
    private String getValueName(FSMvVariable var, String value)
    {
        return getVariableName(var)+"_"+value;
    }
}
