/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package fsmvVerifier.generator;

import fsmv.FSMv;
import fsmv.FSMvRho;
import fsmv.FSMvVariable;
import fsmv.phi.Configuration;
import fsmv.phi.Phi;
import fsmv.predicate.Predicate;
import java.io.FileWriter;
import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.regex.Pattern;

/**
 *
 * @author jvmillo
 */
public class QBFGenerator 
{
    private final String selectedDirPath;
    
    //From fsmv feature.variable.value to qbf variables.
    private final Map<String, String> qbfDesVariables;
    private final Map<String, String> qbfReqVariables;
    
    private final Set<Phi> listOfPhi;
    private final FSMvRho rho_d;
    private final FSMvRho rho_r;
            
    public QBFGenerator(String selectedDirPath, Set<Phi> listOfPhi, FSMvRho rho_d, FSMvRho rho_r)
    {
        this.selectedDirPath= selectedDirPath;
        this.qbfDesVariables = new HashMap<>();
        this.qbfReqVariables = new HashMap<>();
        
        this.listOfPhi=listOfPhi;
        this.rho_d=rho_d;
        this.rho_r=rho_r;
    }
    
    public Map<String, String> getQBFDesVariables(){return qbfDesVariables;}
    public Map<String, String> getQBFReqVariables(){return qbfReqVariables;}


    public void generateQBFFile()
    {
        String designValuesMutualExclusion = buildQbfVariables(true, qbfDesVariables);

        String requirValuesMutualExclusion = buildQbfVariables(false, qbfReqVariables);

        String designRhos=buildRhos(true, qbfDesVariables);

        String reqRhos = buildRhos(false, qbfReqVariables);

        String bigPhiFormula = buildBigPhi(qbfDesVariables, qbfReqVariables);

        String compDesConst = translateCompositionalConstraintsInQBF(qbfDesVariables,true);

        String compReqConst = translateCompositionalConstraintsInQBF(qbfReqVariables,false);

        designRhos=designValuesMutualExclusion+" & "+designRhos;

        reqRhos = requirValuesMutualExclusion+" & "+reqRhos;

        generateQBFFile(qbfDesVariables, qbfReqVariables, designRhos, reqRhos, bigPhiFormula, compDesConst, compReqConst);
    }
        
        
	/**
     * To build the map from {@link FSMvVariable} to QBF
     * @param design
     * @return The mutual exclusive constraints as a string
     */
    private String buildQbfVariables(boolean design, Map<String,String> qbfVariables) 
    {
        long desVarCounter = 0;
        
        String designValuesMutualExclusion="";
        
        String delim="";
        for (Phi phi : listOfPhi) 
        {
        	FSMv fsmv;
        	if(design)
        		fsmv=phi.getDesign();
        	else fsmv=phi.getRequirement();
            //Associate FSMd feature.variable.value with d<number> qbf variables
            for (FSMvVariable fSMvVariable: fsmv.getVariables()) 
            {
                long start = desVarCounter + 1;
                for (String value: fSMvVariable.getQualifiedValues()) 
                {
                    desVarCounter++;
                    qbfVariables.put(value, (design?"d":"r") + desVarCounter);
                }
                designValuesMutualExclusion += delim;
                designValuesMutualExclusion += generateMutualExclusionConstraint(design?"d":"r", start, desVarCounter);
                delim=" & ";
            }
        }
        return designValuesMutualExclusion;
	}
    

	/**
     * To generate the mutual exclusion between the QBF variables that matches 
     * the different values of the same {@link FSMvVariable}
     * @param str Either "d" or "r" for design and req 
     * @param start The starting index of the concerned QBF variables
     * @param varCounter the number of concerned QBF variables(=number of values of the 
     * corresponding {@link FSMvVariable})
     * @return the formula
     */
	private String generateMutualExclusionConstraint(String str, long start, long varCounter) {
        StringBuilder varConst = new StringBuilder("(");
        String upperDelim="";
        for (long j = start; j <= varCounter; j++) 
        {
        	varConst.append(upperDelim);
            varConst.append("(");
        	String delim="";
            for (long i = start; i <= varCounter; i++) 
            {
                varConst.append(delim);
                if (i == j) 
                {
                    varConst.append(" ");
                    varConst.append(str);
                    varConst.append(i);
                } else 
                {
                    varConst.append(" !");
                    varConst.append(str);
                    varConst.append(i);
                }
            	delim=" & ";
            }
            varConst.append(")");
            upperDelim=" | ";            
        }
        varConst.append(")");
        return varConst.toString();
    }
    
    /**
     * To build a QBF formula from the all the rhos of the machines
     * @param design true when building the rhos from the requirements, false when the design
     * @param qbfVariables the map from FSMv variables to QBF variables
     * @return the formula as a string
     */
    private String buildRhos(boolean design, Map<String, String> qbfVariables) 
    {
    	StringBuilder designRhos = new StringBuilder("(");
        String main_delim="";
        for (Phi phi : listOfPhi) 
        {
            FSMv machine=design?phi.getDesign():phi.getRequirement();
        	
            for (Predicate predicate: machine.getRho().getConjuncts()) 
            {
                if(predicate.getEqu().equals("")||Pattern.compile("\\s*\\(?\\s*(true|false)\\s*\\)?\\s*").matcher(predicate.getEqu()).matches())
               /*     ||predicate.getEqu().trim().equals("true")
                    ||predicate.getEqu().trim().equals("false")
                    ||predicate.getEqu().trim().equals("(false)")
                    ||predicate.getEqu().trim().equals("(true)"))*/
                    continue;
                String predicateFormula = predicateToFormula(predicate.toString(), machine.getName(), design, qbfVariables);
                
                if(predicateFormula==null)
                    System.out.println("QBFGenerator.buildRhos(): The following predicate is null:"+predicate.toString());
                
                designRhos.append(main_delim);
                designRhos.append(predicateFormula);
                main_delim=" & ";
            }
        }
        designRhos.append(")");
        
		return designRhos.toString();
	}
    
    /**
     * To build the QBF Phi formula from the {@link Phi}.
     * @param qbfDesVariables The map from FSMd variables to QBF variables
     * @param qbfReqVariables The map from FSMr variables to QBF variables
     * @return
     */
    private String buildBigPhi(Map<String, String> qbfDesVariables, Map<String, String> qbfReqVariables)
    {
        String main_delim="";
    	StringBuilder bigPhi = new StringBuilder("(");
    	for (Phi phi : listOfPhi) 
        {
            
            //Browse all the Phi couple for a given pair FSMd FSMr
            for (Configuration config: phi.getDesign().getListOfValidConfiguration()) 
            {
            	StringBuilder phiFormula = new StringBuilder("( !(");
                String inner_loop_delim="";
                
                for (Map.Entry<String, String> variable : config.getListOfVariable_value().entrySet()) 
                {
                	phiFormula.append(inner_loop_delim);
                    phiFormula.append(qbfDesVariables.get(variable.getKey()+"."+variable.getValue()));
                    inner_loop_delim=" & ";
                }
                
                List<Configuration> correspondingConf=phi.getCorrespondancesTo(config);
                
                phiFormula.append(") |");
                
                if(correspondingConf.size()>=1)
                    phiFormula.append("(");
                phiFormula.append("(");
                
                String intermed_delim="";
                for(Configuration req_conf: correspondingConf)
                {
                    inner_loop_delim="";
                    phiFormula.append(intermed_delim);
                    for (Map.Entry<String, String> variable : req_conf.getListOfVariable_value().entrySet()) 
                    {
                        phiFormula.append(inner_loop_delim);
                        phiFormula.append(qbfReqVariables.get(variable.getKey()+"."+variable.getValue()));
                        inner_loop_delim=" & ";
                    }
                    intermed_delim=" | ";
                
                
                }
                if(correspondingConf.size()>=1)
                    phiFormula.append(")");
                
                phiFormula.append("))");
                bigPhi.append(main_delim);
                bigPhi.append(phiFormula);
                main_delim=" & ";
            }
        }
        bigPhi.append(" )");
        
		return bigPhi.toString();
    }
    
    /**
     * To generate the composition constraints in QBF
     * @param qbfVariables  The map from FSMd variables to QBF variables
     * @param design true when the design constraint is generated, false when the requirement
     * @return
     */
    private String translateCompositionalConstraintsInQBF(Map<String, String> qbfVariables, boolean design)
    {
        FSMvRho rho=design?this.rho_d:this.rho_r;
        
        String compositionConstraint = "(";
       
        String delim="";
        
        for(Predicate predicate: rho.getConjuncts())
        {
            if(predicate.getEqu()==null)
                continue;
            if(predicate.getEqu().equals("")
                    ||predicate.getEqu().trim().equals("true")
                    ||predicate.getEqu().trim().equals("false")
                    ||predicate.getEqu().trim().equals("(false)")
                    ||predicate.getEqu().trim().equals("(true)")
                            )
                continue;
            
            compositionConstraint +=delim;
            compositionConstraint += predicateToFormula(predicate.getEqu(), null, design, qbfVariables);
        	delim=" & ";
        }
        
        if(compositionConstraint.equals("("))
        {
        	compositionConstraint="";
        }
        else
        {
        	compositionConstraint += ")";
        }
        /* 
        if(design)
            System.out.println("CompositionalDesign:"+compositionConstraint);
        else
            System.out.println("CompositionalReq:"+compositionConstraint);
         */
        
        return compositionConstraint;
    }

	/**
	 * To convert a predicate from the FSMv format to QBF one. 
	 * @param predicateStr The FSMv predicate
	 * @param featureName The name of the feature
	 * @param qbfVariables The map from FSMv variables to QBF variables
	 * @return
	 */
    private String predicateToFormula(String predicateStr, String featureName, boolean design, Map<String, String> qbfVariables) 
    {
        predicateStr = predicateStr.trim();
        String predicateFormula = "";
        if (predicateStr.length()>=3&&"and".equalsIgnoreCase(predicateStr.substring(0, 3))) {
            int nextComma = findNextComma(predicateStr);
            predicateFormula += "(";
            predicateFormula += predicateToFormula(predicateStr.substring(4, nextComma), featureName, design, qbfVariables);
            predicateFormula += " & ";
            predicateFormula += predicateToFormula(predicateStr.substring(nextComma + 1, predicateStr.length() - 1), featureName, design, qbfVariables);
            predicateFormula += ")";
        } else if (predicateStr.length()>=3&&"not".equalsIgnoreCase(predicateStr.substring(0, 3))) {
            predicateFormula += "!(";
            predicateFormula += predicateToFormula(predicateStr.substring(4, predicateStr.length() - 1), featureName, design, qbfVariables);
            predicateFormula += ")";
        } else if (predicateStr.length()>=7&&"implies".equalsIgnoreCase(predicateStr.substring(0, 7))) {
            int comma = findNextComma(predicateStr);
            predicateFormula += "( !";
            predicateFormula += predicateToFormula(predicateStr.substring(8, comma), featureName, design, qbfVariables);
            predicateFormula += " | ";
            predicateFormula += predicateToFormula(predicateStr.substring(comma + 1, predicateStr.length() - 1), featureName, design, qbfVariables);
            predicateFormula += ")";
        } else if (predicateStr.length()>=2&&"or".equalsIgnoreCase(predicateStr.substring(0, 2))) {
            int comma = findNextComma(predicateStr);
            predicateFormula += "(";
            predicateFormula += predicateToFormula(predicateStr.substring(3, comma), featureName, design, qbfVariables);
            predicateFormula += " | ";
            predicateFormula += predicateToFormula(predicateStr.substring(comma + 1, predicateStr.length() - 1), featureName, design, qbfVariables);
            predicateFormula += ")";
        } else if (predicateStr.equals("true")||predicateStr.equals("false")||predicateStr.equals("(false)")||predicateStr.equals("(true)")) {  
            return predicateStr;
            //return Pattern.compile("\\(?true\\)?").matcher(predicateStr).matches()?"1":"0";
        } else {
            int comma = findNextComma(predicateStr);
            if (comma == 0) 
            {
                //String[] strArrTemp = predicateStr.split("=");
                //predicateStr=predicateStr.replaceAll("\\s*\\(?(.+)=(.+)\\)?\\s*", "$1.$2");
                predicateStr=predicateStr.replace("=",".");
                if(predicateStr.startsWith("(")&&predicateStr.endsWith(")"))
                    predicateStr=predicateStr.substring(1, predicateStr.length()-1);
                
                predicateFormula = qbfVariables.get(predicateStr);
                
                if(predicateFormula==null)
                    System.out.println("QBFGenerator.predicateToFormula(): Error, unknown variable: "+predicateStr);
                
            } else 
            {
                try 
                {
                    throw new Exception("featureName="+featureName+" design="+design+": Invalid Rho");
                } catch (Exception e) {
                    System.out.println(e.getMessage());
                }
            }
        }
        return predicateFormula;
    }
    
    /**
     * To find the next appropriate comma in the string
     * For example: findNextComma("and(or(a,b),c)") returns 11.
     * @param predicateStr the concerted string
     * @return the index of the next comma
     */
    private int findNextComma(String predicateStr) 
    {
        int paran = 0;
        int i;
        for (i = 0; i < predicateStr.length(); i++) {
            if (predicateStr.charAt(i) == '(') {
                paran++;
            } else if (predicateStr.charAt(i) == ')') {
                paran--;
            } else if (predicateStr.charAt(i) == ',' && paran == 1) {
                break;
            }
        }
        if (i == predicateStr.length()) {
            return 0;
        }
        return i;
    }

    /**
     * To generate the QBF file
     * @param qbfDesVariables The map from FSMd variables to QBF variables
     * @param qbfReqVariables The map from FSMr variables to QBF variables
     * @param designRhos The rhos from the design
     * @param reqRhos  The rhos from the requirement
     * @param bigPhiFormula The phi constraint
     * @param compDesConst the composition design constraint
     * @param compReqConst the composition requirement constraint
     */
	private void generateQBFFile(Map<String, String> qbfDesVariables, Map<String, String> qbfReqVariables, String designRhos, String reqRhos, String bigPhiFormula, String compDesConst, String compReqConst) 
	{
		//Generation of temp.qbf
        try {
            FileWriter fw = new FileWriter(selectedDirPath+"/tmp/temp.qbf");
            fw.write("forall [ ");
            /*
            for (Map.Entry<String, String> entry : qbfDesVariables.entrySet()) 
            {
                fw.write(entry.getValue() + " ");
            }
             */
            for(int i=1;i<=qbfDesVariables.size();i++)
            {
                fw.write("d"+i+" ");
            }
            
            //fw.write("] ( !(" + desConstraints + " & " + designRho + ") | ( exists[");
            fw.write("] ( !("+designRhos);
            if(!compDesConst.equals(""))
            	fw.write(" & "+ compDesConst);
            fw.write(") | ( exists[");
            /*
            for (Map.Entry<String, String> entry : qbfReqVariables.entrySet()) {
                fw.write(entry.getValue() + " ");
            }
             */
            for(int i=1;i<=qbfReqVariables.size();i++)
            {
                fw.write("r"+i+" ");
            }
            
            fw.write("] (");
            //fw.write("( " + reqConstraints + " & " + reqRho + " )");
            fw.write("( " + reqRhos + " )");
            if(!compReqConst.equals(""))
            	fw.write(" & "+compReqConst);
            fw.write(" & " + bigPhiFormula);
            fw.write(" )))");
            fw.close();
            //JOptionPane.showMessageDialog(mainPanel, "File Saved successfully.");
        } catch (IOException ex) {
            //Logger.getLogger(FSMvGUIView.class.getName()).log(Level.SEVERE, null, ex);
        }
		
	}
        
}
