/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package fsmvVerifier.generator;

import fsmv.FSMv;
import fsmv.FSMvEvent;
import fsmv.FSMvVariable;
import fsmv.phi.Configuration;
import fsmvVerifier.generator.promela.ChannelGenerator;
import fsmvVerifier.generator.promela.ChannelSenderGenerator;
import fsmvVerifier.generator.promela.ModelGenerator;
import fsmvVerifier.generator.promela.RunStatementGenerator;
import fsmvVerifier.generator.promela.StateGenerator;
import fsmvVerifier.generator.promela.StateVariableGenerator;
import fsmvVerifier.generator.promela.ValuesGenerator;
import fsmvVerifier.generator.promela.VariableGenerator;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

/**
 *
 * @author jvmillo
 */
public class SingleVariantPromelaGenerator 
{
    private final List<FSMv> fsmds;
    private final Configuration conf;
    private final String file;
    private final List<FSMvEvent> events;
    private int order=0;
    private final String neverClaim;
    
    public SingleVariantPromelaGenerator(String file, List<FSMv> fsmds, Configuration conf, String property)
    {
        this.fsmds=fsmds;
        this.conf=conf;
        this.file=file;
        events=generateListEvents();
        this.neverClaim=property;
    }
    
    public boolean generate()
    {
        try 
        {
            try (FileWriter filew = new FileWriter(new File(file))) {
                filew.write(generatePromelaCode());
            }
            return true;
        } catch (IOException e) {
            //System.out.println(e);
            return false;
        }
    }
    
    public String generatePromelaCode()
    {
        StringBuilder text=new StringBuilder();
        
        //Generate events
        generateEvent(text);
        
        for(FSMv fsmd: fsmds)
            new ValuesGenerator().run(text, fsmd);
        
        for(FSMv fsmd: fsmds)
            new StateGenerator().run(text, fsmd);
        
        for(FSMv fsmd: fsmds)
            new ChannelGenerator().run(text, fsmd);
        
        for(FSMv fsmd: fsmds)
            new StateVariableGenerator().run(text, fsmd);
        
        for(FSMv fsmd: fsmds)
            new VariableGenerator().run(text, fsmd);
        
        //Define the flag to statically schedule the proctype execution
        text.append("/*The flag is used to statically schedule the proctypes*/\n");
        text.append("byte flag;\n");

        text.append("\n\n");
        
        //The environment proctype
        this.generateEnvironmentModel(text);
        
        ModelGenerator mg1=new ModelGenerator(order,fsmds.size()+1);
        for(FSMv fsmd: fsmds)
        {
            mg1.run(text, fsmd);
        }
        
        order=mg1.getOrder();
        assert(order==0);
        
        //this.generateNeverClaim(text);
        
        text.append(neverClaim);
        text.append("\n\n");
        
        this.generateInitBlock(text);
        
        return text.toString();
    }

    private List<FSMvEvent> generateListEvents() 
    {
        List<FSMvEvent> events=new ArrayList<>();
        for(FSMv fsmd: fsmds)
            for(FSMvEvent event: fsmd.getEvents())
                if(!events.contains(event))
                    events.add(event);
        
        return events;
    }
    
    
    private void generateEvent(StringBuilder text)
    {
        
        text.append("/*The Events*/\n");
        for(int i=0;i<events.size();i++)
            text.append("#define  evt_").append(events.get(i).getEvent()).append(" ").append(i).append("\n");
        
        text.append("\n\n");
    }

    private void generateEnvironmentModel(StringBuilder text) 
    {
        
	text.append("proctype environmentModel(){ \n  do \n	::flag==");
        text.append(this.order);
        this.order++;
        text.append(" -> flag=");
        text.append(this.order);
        
        text.append("; atomic{if \n");
        for (int i = 0; i < events.size(); i++) 
        {
            text.append("	::(1)-> ");
            for(FSMv fsmd:fsmds)
            {
                new ChannelSenderGenerator(events.get(i)).run(text, fsmd);
                text.append("  ");
            }
            text.append("\n");
        }
        text.append("\n	fi;}\n  od; \n };\n\n\n");
    }
/*
    private void generateNeverClaim(StringBuilder text) 
    {
        //text.append("\n\n /*never claim definition* /\n");
        text.append("never { \n TO_init: \n	if\n 	::(flag==");
        text.append(this.order);
        text.append(" && ");
        text.append("(");
        text.append(this.neverClaim);
        text.append(")");
        text.append(")");
        text.append("->");
        text.append(" goto accept_S9 \n	::(1) -> goto TO_init \n	fi;\n");
        text.append("accept_S9:\n	if\n	::(1) -> goto TO_init \n	fi; \n } \n\n\n\n");
    }
*/
    private void generateInitBlock(StringBuilder text) 
    {
        text.append("init{ \n flag=0; \n");
        
        //Set a random design configuration amongst the valid configurations
        generateVariableInitBlock(text);
        
        //Launch
        text.append("	atomic { \n");
        text.append("		run environmentModel(); \n");
       
        for(FSMv fsmd: fsmds)
            new RunStatementGenerator().run(text, fsmd);
       
        text.append("		}\n");
       text.append("	}");
    }
    
    private void generateVariableInitBlock(StringBuilder text) 
    {
        for(FSMv fsmd: fsmds)
        {
            for (int j = 0; j < fsmd.getVariables().size(); j++) {
                    text.append(getVariableName(fsmd.getVariables().get(j)))
                            .append("=")
                        .append(getValueName(fsmd.getVariables().get(j), conf.get(fsmd.getVariables().get(j).getQualifiedName())))
                            .append("; \n");
                }
        }
    }
    
    private String getVariableName(FSMvVariable var)
    {
        return var.getQualifiedName().replace(".", "_");
    }
    
    private String getValueName(FSMvVariable var, String value)
    {
        return getVariableName(var)+"_"+value;
    }
}
