package fsmv.util;

import javax.swing.tree.TreeModel;
import javax.swing.tree.TreePath;
import javax.swing.event.TreeModelListener;
import javax.swing.event.EventListenerList;
import java.util.List;
import java.util.ArrayList;
import java.util.Map;
import java.util.HashMap;
import java.io.File;
import java.io.Serializable;

public class FileTreeModel
        implements TreeModel, Serializable, Cloneable {

    /**
	 * 
	 */
	private static final long serialVersionUID = -6354465818270035255L;
	protected EventListenerList listeners;
    private static final Object LEAF = new Serializable() {

		/**
		 * 
		 */
		private static final long serialVersionUID = 214357814716672948L;
    };
    private Map<File,Object> map;
    private File root;

    public FileTreeModel(File root) {
        this.root = root;

        if (!root.isDirectory()) {
            map.put(root, LEAF);
        }

        this.listeners = new EventListenerList();

        this.map = new HashMap<>();
    }

    public Object getRoot() {
        return root;
    }

    public boolean isLeaf(Object node) {
        return map.get(node) == LEAF;
    }

    public int getChildCount(Object node) {
        List<File> children = children(node);

        if (children == null) {
            return 0;
        }

        return children.size();
    }

    public Object getChild(Object parent, int index) {
        return children(parent).get(index);
    }

    public int getIndexOfChild(Object parent, Object child) {
        return children(parent).indexOf(child);
    }

    protected List<File> children(Object node) {
        File f = (File) node;

        Object value = map.get(f);

        if (value == LEAF) {
            return null;
        }

        @SuppressWarnings("unchecked")
		List<File> children = (List<File>) value;

        if (children == null) {
            File[] c = f.listFiles();

            if (c != null) {
                children = new ArrayList<File>(c.length);

                for (int len = c.length, i = 0; i < len; i++) {
                    //String str = c[i].getName();
                    //System.out.println("str :"+str);
                    children.add(c[i]);
                    if (!c[i].isDirectory()) {
                        map.put(c[i], LEAF);
                    }
                }
            } else {
                children = new ArrayList<File>(0);
            }

            map.put(f, children);
        }

        return children;
    }

    public void valueForPathChanged(TreePath path, Object value) {
    }

    public void addTreeModelListener(TreeModelListener l) {
        listeners.add(TreeModelListener.class, l);
    }

    public void removeTreeModelListener(TreeModelListener l) {
        listeners.remove(TreeModelListener.class, l);
    }

    public Object clone() {
        try {
            FileTreeModel clone = (FileTreeModel) super.clone();

            clone.listeners = new EventListenerList();

            clone.map = new HashMap<File,Object>(map);

            return clone;
        } catch (CloneNotSupportedException e) {
            throw new InternalError();
        }
    }

    /*
    public static void main(String[] args)
    {
    args = new String[1];
    args[0]="E:\\software";
    if (args.length != 1)
    {
    System.err.println("Usage: java FileTreeModel path");
    System.exit(1);
    }

    File root = new File(args[0]);

    if (!root.exists())
    {
    System.err.println(root+ ": No such file or directory");
    System.exit(2);
    }

    JTree tree = new JTree(new FileTreeModel(root));

    JFrame f = new JFrame(root.toString());

    f.addWindowListener(new WindowAdapter()
    {
    public void windowClosing(WindowEvent e)
    {
    System.exit(0);
    }
    });

    f.getContentPane().add(new JScrollPane(tree));

    f.pack();
    f.setVisible(true);
    }
     */
}
