/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package fsmv.util;

import java.io.File;

import javax.swing.JPanel;
import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.MutableTreeNode;
import javax.swing.tree.TreeModel;

/**
 * Example which displays the directory structure of the system
 * in a JTree coponent. The nodes are expandable, and will list
 * the sub directories of the selected node.
 *
 * @author Havard Rast Blok
 *
 */
public class FileTreeProModel extends JPanel {

    /**
	 * 
	 */
	private static final long serialVersionUID = -8440322732188134485L;

	/** The model which holds the nodes */
    protected TreeModel model;

    /** The tree GUI */
    protected JTree tree;

    /**
     * Constructs the panel which holds a JTree with the file structure.
     */
    public FileTreeProModel(File file) {

        //get all nodes for top file systems or disks
        //On Linux/Unix this will only be '/'
        //while on Window there will typically be more: 'A:', 'C:' etc.
        File roots[] = File.listRoots();
        //create the top node
        MutableTreeNode root = new DefaultMutableTreeNode();
        //loop through all these nodes and add them to the root Computer node
        int i = 0;
        for(File f : roots) {
            DefaultMutableTreeNode node = new DefaultMutableTreeNode(f.getAbsoluteFile().toString());
            root.insert(node, i++);
        }

        //create a tree model with the Computer node as root
        model = new DefaultTreeModel(root);

        //create a JTree GUI component with the model to display
        tree = new JTree(model);

        //add a selection listener, to listen for selected nodes
        tree.addTreeSelectionListener(new MyTreeSelectionListener());
    }

    /**
     * Adds the sub directories, if any, of the given path to the given node.
     *
     * @param node tree node to add sub directory nodes to
     * @param path file system path to search for sub directories
     */
    protected static void addChildren(MutableTreeNode node, String path) {
        //create filter for directories only
        DirectoryFileFilter filter = new DirectoryFileFilter();
  
        //get directories
        File dir = new File(path);
        File subDirs[] = dir.listFiles(filter);

        //add sub nodes to given nodes
        if (subDirs != null) {
            int i = 0;
            for (File f : subDirs) {
                MutableTreeNode child = new DefaultMutableTreeNode(f.getName());
                node.insert(child, i++);
            }
        }
    }
}