package lucci.io.stream;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

import toools.io.JavaResource;

public class JavaResourceStreamSource extends AbstractStreamSource
{
	private JavaResource resource = null;

	public String getName()
	{
		return getJavaResource().getPath();
	}

	public boolean isReadable()
	{
		InputStream is = resource.getInputStream();

		if ( is == null )
		{
			return false;
		}
		else
		{
			try
			{
				is.close();
			}
			catch ( IOException ex )
			{
			}

			return true;
		}
	}

	public boolean isWritable()
	{
		return false;
	}

	public InputStream createInputStreamImpl()
		throws IOException
	{
		return resource.getInputStream();
	}

	public OutputStream createOutputStreamImpl()
		throws IOException
	{
		throw new IOException( "resource " + resource.getPath() + " cannot be written" );
	}

	/**
	 * @return the resource.
	 */
	public JavaResource getJavaResource()
	{
		return resource;
	}


	/**
	 * Sets the resource.
	 */
	public void setJavaResource( JavaResource resource )
	{
		if ( resource == null )
			throw new IllegalArgumentException( "java resource cannot be set to null" );

		this.resource = resource;
	}


	/**
	 * Sets the resource name for this document.
	 * resource:///ref
	 *
	 *
	 */
	public void setAsText( StreamSource refStreamSource, String name )
	{
		boolean isRelative = isRelativeTo( refStreamSource, name );

		if ( name.startsWith( "resource://" ) )
		{
			name = name.substring( 11 );
		}

		if ( isRelative )
		{
			JavaResource ref = ((JavaResourceStreamSource) refStreamSource).getJavaResource();
			String refName = ref.getPath();
			int pos = refName.lastIndexOf( '/' );

			if ( pos == -1 )
			{
				setJavaResource( new JavaResource( name ) );
			}
			else
			{
				setJavaResource( new JavaResource( refName.substring( 0, pos + 1 ) + name ) );
			}
		}
		else
		{
			setJavaResource( new JavaResource( name ) );
		}
	}

	private boolean isRelativeTo( StreamSource ref, String name )
	{
		if ( ref == null )
		{
			return false;
		}
		else if ( !(ref instanceof JavaResourceStreamSource) )
		{
			return false;
		}
		else if ( name.startsWith( "resource://" ) )
		{
			return false;
		}
		else if ( name.startsWith( "/" ) )
		{
			return false;
		}
		else
		{
			return true;
		}
	}

	public String getProtocol()
	{
		return "resource";
	}
}
