package lucci.io.stream;

import java.io.*;


public interface StreamSource
{
	/**
	 * @return the name of the stream source. If it's based on a
	 * file, the name of the file is returned, if it is based on
	 * a Java resource, the name of this resource is returned.
	 * The name may be null if the stream source is not based on
	 * a named object, for example a string buffer has no name.
	 */
	String getName();

	/**
	 * @return if the data source is compressed.
	 */
	boolean isCompressed();

	/**
	 * @return if the resource is readable.
	 */
	boolean isReadable();

	/**
	 * @return if the resource is writable.
	 */
	boolean isWritable();

	/**
	 * @return the input stream for the document.
	 */
	InputStream createInputStream()
		throws IOException;

	/**
	 * @return the output stream for the document.
	 */
	OutputStream createOutputStream()
		throws IOException;

	/**
	 * @return a byte array of the data contained in the source.
	 * This is a convenience method.
	 */
	byte[] readItAll() throws IOException;

	/**
	 * Sets the data into the source from the given byte array.
	 * This is a convenience method.
	 */
	void writeItAll(byte[] bytes) throws IOException;

	/**
	 * Sets the resource as a string, relatively to the given resource.
	 */
	void setAsText( StreamSource reference, String name );

	/**
	 * @return the protocol for the data source.
	 */
	String getProtocol();
}
