package lucci.io.stream;

import java.io.*;

public class StringBufferStreamSource extends AbstractStreamSource
{
	/* the source buffer */
	private StringBuffer buffer = null;

	public StringBufferStreamSource()
	{
	    this( new StringBuffer() );
	}

	public StringBufferStreamSource( StringBuffer buffer )
	{
	    setStringBuffer( buffer );
	}


	public boolean isReadable()
	{
		return true;
	}

	public boolean isWritable()
	{
		return true;
	}

	public String getName()
	{
		return null;
	}

	public boolean isCompressed()
	{
		return false;
	}

	public InputStream createInputStreamImpl()
		throws IOException
	{
		return new StringBufferInputStream( getStringBuffer().toString() );
	}

	public OutputStream createOutputStreamImpl()
		throws IOException
	{
	        buffer.setLength( 0 );
		return new StringBufferOutputStream( buffer );
	}

	public byte[] readItAll()
		throws IOException
	{
		return getStringBuffer().toString().getBytes();
	}



	public void setAsText( StreamSource ref, String s )
	{
		setStringBuffer( new StringBuffer( s ) );
	}

	/**
	 * @return the string buffer for the document.
	 */
	public StringBuffer getStringBuffer()
	{
		return buffer;
	}


	/**
	 * Sets the buffer for the document.
	 */
	public void setStringBuffer( StringBuffer buffer )
	{
		if ( buffer == null )
			throw new IllegalArgumentException( "string buffer cannot be set to null" );

		this.buffer = buffer;
	}

	private class StringBufferOutputStream extends OutputStream
	{
		private StringBuffer buffer;

		public StringBufferOutputStream( StringBuffer buffer )
		{
			this.buffer = buffer;
		}

		public void write( int c )
			throws IOException
		{
			buffer.append( (char) c );
		}
	}

	public String getProtocol()
	{
		return null;
	}
}
