package lucci.io.stream;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;


public class URLStreamSource extends AbstractStreamSource
{
	/* the source url */
	private URL url = null;

	public boolean isReadable()
	{
		return true;
	}

	public boolean isWritable()
	{
		return true;
	}


	public InputStream createInputStreamImpl()
		throws IOException
	{
		return getURL().openConnection().getInputStream();
	}

	public OutputStream createOutputStreamImpl()
		throws IOException
	{
		URLConnection connection = getURL().openConnection();
		connection.setDoOutput( true );
		return connection.getOutputStream();
	}


	public String getName()
	{
		return url.toExternalForm();
	}


	/**
	 * @return the URL for the document.
	 */
	public URL getURL()
	{
		return url;
	}


	/**
	 * Sets the URL for the document.
	 */
	public void setURL( URL url )
	{
		if ( url == null )
			throw new IllegalArgumentException( "url cannot be set to null" );

		this.url = url;
	}


	/**
	 * Sets the filename for this document.
	 */
	public void setAsText( StreamSource refStreamSource, String url )
	{
		boolean isRelative = isRelativeTo( refStreamSource, url );

		if ( isRelative )
		{
			try
			{
				URL refUrl = ((URLStreamSource) refStreamSource).getURL();
				setURL( new URL( refUrl, url ) );
			}
			catch ( MalformedURLException ex )
			{
				throw new IllegalArgumentException( url + " is a malformed URL" );
			}
		}
		else
		{
			setAsText( url );
		}
	}

	private boolean isRelativeTo( StreamSource ref, String url )
	{
		if ( ref == null )
		{
			return false;
		}
		else if ( !(ref instanceof URLStreamSource) )
		{
			return false;
		}
		else
		{
			return true;
		}
	}



	private void setAsText( String name )
	{
		try
		{
			setURL( new URL( name ) );
		}
		catch ( MalformedURLException ex )
		{
			throw new IllegalArgumentException( name + " is a malformed URL" );
		}
	}

	public String getProtocol()
	{
		return getURL().getProtocol();
	}
}
