package toools;

import java.util.Random;

import toools.collections.StopWatch;


public class CA30PRNG 
{
	private final IntCellularAutomata ca = new IntCellularAutomata();
	
	public CA30PRNG()
	{
		this((int) System.currentTimeMillis());
	}

	public CA30PRNG(int seed)
	{
		ca.setCells(seed);
	}

	public boolean nextBit()
	{
		ca.evolve();
		return (ca.cells & 1024) == 0;
	}

	public int nextInt()
	{
		int n = 0;
		
		for (int i = 0; i < 32; ++i)
		{
			n |= (nextBit() ? 0 : 1) << i;
		}
		
		return n;
	}

	public int nextInt2()
	{
		ca.evolve();
		return ca.cells;
	}

	public static void main(String[] args)
	{
		int n = 1000000;
		StopWatch sw = new StopWatch();

		{
			CA30PRNG p = new CA30PRNG();

			for (int i = 0; i < n; ++i)
			{
				p.nextInt2();
			}
			
			System.out.println(sw.getElapsedMillis(true));
		}
		
		{
			Random r = new Random();

			for (int i = 0; i < n; ++i)
			{
				r.nextInt();
			}

			System.out.println(sw.getElapsedMillis(true));
		}
	}
}
