package toools;

import java.net.URL;
import java.net.URLClassLoader;
import java.util.ArrayList;
import java.util.List;

import toools.io.JavaResource;
import toools.io.file.AbstractFile;
import toools.io.file.RegularFile;

/**
 * This represents either a ZIP/JAR file or a directory
 * 
 * @author lhogie
 * 
 */

public class ClassContainer
{
    private final AbstractFile file;
    private final ClassLoader classLoader;

    public ClassContainer(AbstractFile f)
    {
	this(f, null);
    }

    public ClassContainer(AbstractFile f, ClassLoader classLoader)
    {
	if (f == null)
	    throw new NullPointerException();

	this.file = f;
	this.classLoader = classLoader;
    }

    public List<Class<?>> listAllClasses()
    {
	List<Class<?>> classes = new ArrayList<Class<?>>();

	for (JavaResource thisResource : JavaResource.listResources(this))
	{
	    String resname = thisResource.getPath();

	    if (resname.endsWith(".class"))
	    {
		// removes the .class extension
		String classname = resname.substring(0, resname.length() - 6);

		// convert separators from / to .
		classname = classname.replace('/', '.');

		// remove the first .
		if (classname.startsWith("."))
		{
		    classname = classname.substring(1);
		}

		Class<?> clazz = loadClass(classname);

		// if the class could be loaded
		if (clazz != null)
		{
		    classes.add(clazz);
		}
	    }
	}

	return classes;
    }

    public Class<?> loadClass(String classname)
    {
	try
	{
	    if (classLoader == null)
	    {
		return URLClassLoader.newInstance(new URL[] { file.toURL() }).loadClass(classname);
	    }
	    else
	    {
		return URLClassLoader.newInstance(new URL[] { file.toURL() }, classLoader).loadClass(classname);
	    }
	}
	catch (Throwable e)
	{
	    return null;
	}
    }

    @Override
    public String toString()
    {
	return file.getPath();
    }

    public AbstractFile getFile()
    {
	return file;
    }

    public List<Class<?>> findClassesWhoseTheNameMatches(String regexp)
    {
	List<Class<?>> r = new ArrayList<Class<?>>();

	for (Class<?> c : listAllClasses())
	{
	    if (c.getName().matches(regexp))
	    {
		r.add(c);
	    }
	}

	return r;
    }

    public static void main(String[] args)
    {
	ClassContainer cc = new ClassContainer(new RegularFile("~/lib/java/acrobat.jar"));
	Class c = cc.findClassesWhoseTheNameMatches(".*VChange.*").get(0);
	System.out.println(c.getName());
	System.out.println(c.getMethods().length);
	Object o = Clazz.makeInstance(c);
	// System.out.println("instance: "+ o);
    }

}
