package toools;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import toools.io.FileUtilities;
import toools.io.file.AbstractFile;
import toools.io.file.Directory;
import toools.io.file.RegularFile;

public class ClassPath extends ArrayList<ClassContainer>
{
    public static ClassPath retrieveClassPath()
    {
	ClassPath urls = new ClassPath();

	for (String entry : System.getProperty("java.class.path").split(File.pathSeparator))
	{
	    entry = entry.trim();

	    if (!entry.isEmpty())
	    {
		urls.add(new ClassContainer(AbstractFile.map(entry, Directory.class)));
	    }
	}

	return urls;
    }

    public List<Class<?>> listAllClasses()
    {
	List<Class<?>> classes = new ArrayList<Class<?>>();

	for (ClassContainer ce : this)
	{
	    classes.addAll(ce.listAllClasses());
	}

	return classes;
    }

    public List<ClassContainer> getContainer(String re)
    {
	List<ClassContainer> matchingContainers = new ArrayList();

	for (ClassContainer ce : this)
	{
	    if (ce.getFile().getPath().matches(re))
	    {
		matchingContainers.add(ce);
	    }
	}

	return matchingContainers;
    }

    private RegularFile zipFile;
    List<RegularFile> zipFiles;

    public List<RegularFile> getJars() throws IOException
    {
	if (zipFiles == null)
	{
	    zipFiles = new ArrayList();

	    for (ClassContainer cc : this)
	    {
		AbstractFile f = cc.getFile();

		if (f instanceof RegularFile)
		{
		    zipFiles.add((RegularFile) cc.getFile());
		}
		else
		{
		    Directory d = (Directory) f;
		    RegularFile zz = new RegularFile("/tmp/" + d.getPath().replace('/', '_') + ".jar");

		    if (!d.getChildFiles().isEmpty())
		    {
			FileUtilities.zip(zz, d, null);
			zipFiles.add(zz);
		    }
		}
	    }
	}

	return zipFiles;
    }

    public RegularFile getSeftContainedZipFile() throws IOException
    {
	if (zipFile == null)
	{
	    Map<String, byte[]> map = new HashMap<String, byte[]>();

	    for (ClassContainer cc : this)
	    {
		AbstractFile f = cc.getFile();

		if (f instanceof RegularFile)
		{
		    map.put(cc.getFile().getName(), ((RegularFile) cc.getFile()).getContent());
		}
		else
		{
		    RegularFile zz = new RegularFile("/tmp/sdfsdfjskjfshkjhqg");
		    FileUtilities.zip(zz, (Directory) f, null);
		    map.put(f.getPath().replace('/', '-') + ".jar", zz.getContent());
		    zz.delete();
		}
	    }

	    zipFile = new RegularFile("/tmp/" + Math.random() + ".zip");

	    if (zipFile.exists())
	    {
		zipFile.delete();
	    }

	    FileUtilities.zip(zipFile, map);
	}

	return zipFile;
    }

    @Override
    public String toString()
    {
	StringBuilder b = new StringBuilder();

	for (ClassContainer cc : this)
	{
	    b.append(cc.getFile().getPath() + (cc.getFile().exists() ? "" : "(not found)") + "\n");
	}

	return b.toString();
    }

    public int sizeInBytes()
    {
	int s = 0;

	for (ClassContainer cc : this)
	{
	    s += cc.getFile().getSize();
	}

	return s;
    }

}
