package toools;

import java.lang.reflect.Field;
import java.lang.reflect.Modifier;
import java.util.HashMap;
import java.util.Map;

public class Introspector
{

    private final Map<String, Field> fields = new HashMap<String, Field>();

    private Introspector(Class<?> c)
    {
	for (Field f : c.getDeclaredFields())
	{
	    // if this field is neither transient, static, nor excluded by
	    // the user
	    if ((f.getModifiers() & (Modifier.TRANSIENT | Modifier.STATIC)) == 0)
	    {
		f.setAccessible(true);
		fields.put(f.getName(), f);
	    }
	}

	// adds also the fields declared in the superclass, if any
	if (c.getSuperclass() != null)
	{
	    for (Field f : getIntrospector(c.getSuperclass()).getFields().values())
	    {
		fields.put(f.getName(), f);
	    }
	}
    }

    public Map<String, Field> getFields()
    {
	return fields;
    }

    public Object getFieldValue(Field f, Object o)
    {
	try
	{
	    return f.get(o);
	}
	catch (Throwable e)
	{
	    throw new IllegalStateException();
	}
    }

    public void setFieldValue(Field f, Object targetObject, Object value)
    {
	try
	{
	    f.set(targetObject, value);
	}
	catch (Throwable e)
	{
	    e.printStackTrace();
	    throw new IllegalStateException();
	}
    }

    public static final Map<Class<?>, Introspector> map = new HashMap();

    public static Introspector getIntrospector(Class<?> c)
    {
	Introspector b = map.get(c);

	if (b == null)
	{
	    map.put(c, b = new Introspector(c));
	}

	return b;
    }
}
