package toools.beans;

import java.beans.BeanInfo;
import java.beans.IntrospectionException;
import java.beans.Introspector;
import java.beans.PropertyDescriptor;
import java.io.IOException;
import java.io.StringReader;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;

public class Bean
{
    final private Class<?> beanClass;
    final private Map<String, Property> properties = new HashMap<String, Property>();

    public Bean(Class<?> clazz)
    {
	this.beanClass = clazz;

	try
	{
	    BeanInfo i = Introspector.getBeanInfo(beanClass);

	    for (PropertyDescriptor d : i.getPropertyDescriptors())
	    {
		if (d.getReadMethod() != null)
		{
		    Property p = new Property(d);
		    this.properties.put(p.getName(), p);
		}
	    }
	}
	catch (IntrospectionException e)
	{
	    throw new IllegalArgumentException("cannot introspect bean " + clazz.getName());
	}
    }

    public String getName()
    {
	return getBeanClass().getName();
    }

    public Class<?> getBeanClass()
    {
	return beanClass;
    }

    public Map<String, Property> getProperties()
    {
	return properties;
    }

    @Override
    public String toString()
    {
	return getName() + ", properties: " + getProperties().keySet();
    }

    public String propertiesAsText(Object o, String regexp)
    {
	StringBuilder b = new StringBuilder();

	for (Property p : getProperties().values())
	{
	    if (p.getName().matches(regexp))
	    {
		    b.append(p.getName() + "=" + p.getValue(o) + "\n");
	    }
	}

	return b.toString();
    }

    public void setPropertyValueFromString(String propertyValuesText, Object target, String propertyNameFilter, Map<Class<?>, FromToString> ser)
    {
	try
	{
	    Properties t = new Properties();
	    t.load(new StringReader(propertyValuesText));

	    for (Property p : getProperties().values())
	    {
		String propName = p.getName();

		if (p.getName().matches(propertyNameFilter))
		{
		    String value = t.get(propName).toString();
		    Class<?> c = p.getType();

		    if (c == int.class || c == Integer.class)
		    {
			p.setValue(target, Integer.valueOf(value));
		    }
		    else if (c == long.class || c == Long.class)
		    {
			p.setValue(target, Long.valueOf(value));
		    }
		    else if (c == boolean.class || c == Boolean.class)
		    {
			p.setValue(target, Boolean.valueOf(value));
		    }
		    else if (c == double.class || c == Double.class)
		    {
			p.setValue(target, Double.valueOf(value));
		    }
		    else if (c == float.class || c == Float.class)
		    {
			p.setValue(target, Float.valueOf(value));
		    }
		    else if (c == char.class || c == Character.class)
		    {
			if (value.length() == 1)
			{
			    p.setValue(target, value.charAt(0));
			}
			else
			{
			    throw new IllegalArgumentException("invalid character: " + value);
			}
		    }
		    else if (c == String.class)
		    {
			p.setValue(target, value);
		    }
		    else
		    {
			FromToString sers = ser.get(c);
			
			if (sers == null)
			{
				throw new IllegalArgumentException("can't handle class: " + c + " for property " + propName) ;
			}
			else
			{
			//    sers.
			}
		    }
		}

	    }
	}
	catch (IOException e)
	{
	    throw new IllegalStateException();
	}
    }

    public static void main(String... args)
    {
	Bean b = new Bean(Bean.class);
	System.out.println(b);
	System.out.println(b.getProperties().get("name").isReadOnly());
    }
}
