package toools.beans;

import java.beans.PropertyDescriptor;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

public class Property
{
    private PropertyDescriptor descriptor;

    public Property(PropertyDescriptor d)
    {
	this.descriptor = d;
    }

    public String getName()
    {
	return this.descriptor.getName();
    }

    public Class<?> getType()
    {
	return this.descriptor.getPropertyType();
    }

    public boolean isReadOnly()
    {
	return this.descriptor.getWriteMethod() == null;
    }

    public Object getValue(Object bean)
    {
	try
	{
	    return this.descriptor.getReadMethod().invoke(bean, null);
	}
	catch (IllegalArgumentException e)
	{
	    // TODO Auto-generated catch block
	    e.printStackTrace();
	    throw new IllegalStateException();
	}
	catch (IllegalAccessException e)
	{
	    // TODO Auto-generated catch block
	    e.printStackTrace();
	    throw new IllegalStateException();
	}
	catch (InvocationTargetException e)
	{
	    // TODO Auto-generated catch block
	    e.printStackTrace();
	    throw new IllegalStateException();
	}
    }

    public void setValue(Object bean, Object value)
    {
	if (bean == null)
	    throw new IllegalArgumentException("cannot set a value to the null object");

	try
	{
	    Method setter = this.descriptor.getWriteMethod();

	    if (setter == null)
	    {
		throw new IllegalStateException("no setter for property " + getName());
	    }
	    else
	    {
		setter.invoke(bean, new Object[] { value });
	    }
	}
	catch (IllegalArgumentException e)
	{
	    // TODO Auto-generated catch block
	    e.printStackTrace();
	    throw new IllegalStateException();
	}
	catch (IllegalAccessException e)
	{
	    // TODO Auto-generated catch block
	    e.printStackTrace();
	    throw new IllegalStateException();
	}
	catch (InvocationTargetException e)
	{
	    // TODO Auto-generated catch block
	    e.printStackTrace();
	    throw new IllegalStateException();
	}
    }

    @Override
    public String toString()
    {
	return getName() + ' ' + (isReadOnly() ? "(ro)" : "(rw)");
    }

    public void setValueAsString(Object o, String value)
    {
	Class<?> type = getType();

	if (type == int.class)
	{
	    setValue(o, Integer.valueOf(value));
	}
	else if (type == double.class)
	{
	    setValue(o, Double.valueOf(value));
	}
	else if (type == boolean.class)
	{
	    setValue(o, Boolean.valueOf(value));
	}
	else
	{
	    throw new IllegalStateException("cannot convert string to " + type.getName());
	}
    }

}
