package toools.collections;

import java.util.ArrayList;

public class AutoGrowingArrayList<V> extends ArrayList<V>
{
    public AutoGrowingArrayList()
    {
	super(5);
    }

    public AutoGrowingArrayList(int initialCapacity)
    {
	super(initialCapacity);
    }

    @Override
    public void add(int index, V o)
    {
	ensureIndexCanBeStored(index);
	super.add(index, o);
    }

    @Override
    public V get(int i)
    {
	// if the index is within the bounds
	if (i < size())
	{
	    // return the value in the indexed cell
	    return super.get(i);
	}
	else
	{
	    return null;
	}
    }

    @Override
    public V set(int index, V o)
    {
	ensureIndexCanBeStored(index);
	super.set(index, o);
	return o;
    }

    public int ensureIndexCanBeStored(int index)
    {
	if (index < 0)
	    throw new IllegalArgumentException();

	int oldsize = size();

	if (index < oldsize)
	{
	    return 0;
	}
	else
	{
	    ensureCapacity(index);
	    int newsize = index + 1;

	    for (int n = oldsize; n < newsize; ++n)
	    {
		super.add(null);
	    }

	    return newsize - oldsize;
	}
    }

    // returns the number of null elements deleted
    public int shrink()
    {
	int initialSize = size();
	int i = size();

	// if the previous cell is empty, shift the index to it
	while (i > 0 && get(i - 1) == null)
	{
	    --i;
	}

	if (i == 0)
	{
	    clear();
	}
	else
	{
	    removeRange(i, size());
	}
	
	return initialSize - i;
    }
}
