package toools.collections;

import toools.math.Interval;


/*
 * Created on Oct 10, 2004
 */

/**
 * @author luc.hogie
 */
public interface Filter<T>
{
    boolean accept(T o);

    public static class ClassFilter<T> implements Filter<Class<T>>
    {
        private Class<T> clazz;

        public ClassFilter(Class<T> c)
        {
            this.clazz = c;
        }

        public boolean accept(Class<T> o)
        {
            return clazz.isInstance(o);
        }
    }

    public static class FilterObjectByClass<T> implements Filter<Object>
    {
        private Class<T> clazz;

        public FilterObjectByClass(Class<T> c)
        {
            this.clazz = c;
        }

        public boolean accept(Object o)
        {
            return clazz.isAssignableFrom(o.getClass());
        }
    }

    public static class StringFilter implements Filter<String>
    {
        private String re;

        public StringFilter(String re)
        {
            this.re = re;
        }

        public boolean accept(String o)
        {
            return o.matches(re);
        }
    }

    public static class IntervalFilter implements Filter<Double>
    {
        private Interval i;

        public IntervalFilter(Interval i)
        {
            this.i = i;
        }

        public boolean accept(Double o)
        {
            return i.include(o);
        }
    }

}
