/**
 * Grph
 * Initial Software by Luc HOGIE, Issam TAHIRI, Aurélien LANCIN, Nathann COHEN, David COUDERT.
 * Copyright © INRIA/CNRS/UNS, All Rights Reserved, 2011, v0.9
 *
 * The Grph license grants any use or destribution of both binaries and source code, if
 * a prior notification was made to the Grph development team.
 * Modification of the source code is not permitted. 
 * 
 *
 */

package toools.collections;

import java.util.ArrayList;
import java.util.List;

public final class IntQueue
{
    private int size = 0;
    private int first = 0;
    private final List<int[]> blocks = new ArrayList<int[]>();
    private final int blockSize;

    public IntQueue()
    {
	this(256);
    }

    public IntQueue(int blockSize)
    {
	this.blockSize = blockSize;
    }

    public void add(int e)
    {
	int globalIndex = first + size;
	int blockIndex = globalIndex / blockSize;

	// if the index of the block refers to a non-existing block
	if (blockIndex == blocks.size())
	{
	    blocks.add(new int[blockSize]);
	}

	blocks.get(blockIndex)[globalIndex % blockSize] = e;
	++size;
    }

    public int getSize()
    {
	return size;
    }

    public int poll()
    {
	if (size == 0)
	    throw new IllegalStateException("no element to poll");

	// get the value
	int e = blocks.get(0)[first];
	
	// remove it
	++first;
	--size;

	// if the first block is no longer used, remove it
	if (first == blockSize)
	{
	    blocks.remove(0);
	    first = 0;
	}

	return e;
    }

    public void clear()
    {
	blocks.clear();
	first = 0;
	size = 0;
    }

    public static void main(String[] args)
    {
	IntQueue q = new IntQueue(4);

	for (int i = 0; i < 20; ++i)
	{
	    q.add(i);
	}

	while (q.getSize() > 0)
	{
	    int e = q.poll();
	    System.out.println( e);
	}

	System.out.println("done");
    }
}
