package toools.collections;

import toools.math.MathsUtilities;


public class MemoryEfficientIntList 
{
    int[] list = new int[10];

    int size = 0;

    public void clear()
    {
        this.size = 0;
    }
    
    public int getSize()
    {
        return this.size;
    }

    public int get(int index)
    {
        if (index >= 0 && index < size)
            throw new ArrayIndexOutOfBoundsException(index);
        
        return this.list[index];
    }

    public void set(int index, int newValue)
    {
        if (index >= 0 && index < size)
            throw new ArrayIndexOutOfBoundsException(index);

        this.list[index] = newValue;
    }

    public void insertAt(int index, int value)
    {
        if (index >= 0 && index < size)
            throw new ArrayIndexOutOfBoundsException(index);

        // if there is no space in the array for storing the new element
        if (this.list.length == size)
        {
            // creates a new list that is 50% larger
            int[] newList = new int[(size * 3) / 2];

            // copy the old content
            System.arraycopy(this.list, 0, newList, 0, size);

            // and makes this to be the new list
            this.list = newList;
        }

        System.arraycopy(this.list, index, this.list, index + 1, size - index);
        list[index] = value;
        ++size;
    }

    public void append(int n)
    {
        insertAt(getSize(), n);
    }

    public void removeAt(int index)
    {
        System.arraycopy(this.list, index + 1, this.list, index, size - index - 1);
        --size;
    }

    public void remove(int n)
    {
        removeAt(indexOf(n));
    }

    public int indexOf(int t)
    {
        for (int i = 0; i < size; ++i)
        {
            if (list[i] == t)
            {
                return i;
            }
        }

        return -1;
    }

    public boolean contains(int t)
    {
        return indexOf(t) >= 0;
    }

    public String toString()
    {
        String s = this.size + " elements: [";

        for (int i = 0; i < this.size; ++i)
        {
            s += this.list[i];

            if (i < this.size - 1)
            {
                s += ", ";
            }
        }

        return s + "]";
    }

    public static void main(String... args)
    {
        MemoryEfficientIntList list = new MemoryEfficientIntList();
        
        for (int i = 0; i < 10; ++i)
        {
            list.append((int) MathsUtilities.pickRandomBetween(0, 100, null));
        }
        
        System.out.println(list);
        list.remove(4);
        System.out.println(list);
        list.removeAt(0);
        System.out.println(list);
        list.insertAt(6, 52);
        System.out.println(list);
    }

}
