package toools.config;

import java.io.File;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.lang.reflect.InvocationTargetException;
import java.util.Random;

import javax.xml.parsers.ParserConfigurationException;


import org.xml.sax.SAXException;

import toools.ExceptionUtilities;
import toools.io.FileUtilities;
import toools.text.xml.XMLNode;

public class ConfigurationUtilities
{
	/**
	 * Instantiate a class of the given type that takes a configuration as a
	 * constructor parameter.
	 * 
	 * @param configuration
	 * @param clazz
	 * @return
	 */
	public static Object create(Configuration configuration, Class<?> clazz)
	{
		try
		{
			return clazz.getConstructor(configuration.getClass()).newInstance(configuration);
		}
		catch (Throwable e)
		{
			throw new IllegalStateException("cannot instantiate class " + clazz.getName() + " with argument " + configuration.getClass() + ExceptionUtilities.toString(e));
		}
	}

	public static Object createWithRandom(Configuration configuration, Random random, Class<?> clazz)
	{
		try
		{
			return clazz.getConstructor(Configuration.class, Random.class).newInstance(configuration, random);
		}
		catch (IllegalArgumentException e)
		{
			e.printStackTrace();
		}
		catch (SecurityException e)
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		catch (InstantiationException e)
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		catch (IllegalAccessException e)
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		catch (InvocationTargetException e)
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		catch (NoSuchMethodException e)
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		throw new IllegalStateException("instantiation has failed for class: " + clazz.getName());
	}

	public static String xmlToConfiguration(String xml) throws ParserConfigurationException, UnsupportedEncodingException, SAXException, IOException
	{

		XMLNode root = toools.text.xml.XMLUtilities.xml2node(xml, false);

		if (!root.getName().equals("configuration")) throw new IllegalArgumentException("root element shoudl be <configuration>");

		StringBuilder buf = new StringBuilder();

		for (XMLNode node : root.getChildren())
		{
			if (!node.getName().equals("#text"))
			{
				buf.append(xml(node));
			}
		}

		return buf.toString();

	}

	private static String xml(XMLNode node)
	{
		StringBuilder buf = new StringBuilder();
		buf.append("section " + node.getName() + "\n");

		for (String k : node.getAttributes().keySet())
		{
			buf.append(k + " = {" + node.getAttributes().get(k) + "}\n");
		}

		for (XMLNode n : node.getChildren())
		{
			buf.append(xml(n));
		}

		buf.append("end of section\n");
		return buf.toString();
	}

	public static void main(String... args) throws IOException, ParserConfigurationException, SAXException
	{
		System.out.println(FileUtilities.getCurrentDirectory());
		String xml = new String(FileUtilities.getFileContent(new File("src/lucci/config/example.xml")));
		System.out.println(xmlToConfiguration(xml));
	}
}
