package toools.config;


import java.io.File;
import java.io.FileOutputStream;
import java.io.FileReader;

import toools.math.relation.HashRelation;
import toools.math.relation.Relation;
import toools.math.relation.RelationIO;



public class PersistingPreferences 
{
    static Relation<Integer, Relation<String, String>> r = new HashRelation<Integer, Relation<String, String>>();

    public static Relation<String, String> loadPreferencesBelongingTo(Object object)
    {
        try
        {
            File file = getFile(object);

            if (file.exists())
            {
                FileReader fis = new FileReader(file);
                Relation<String, String> r = RelationIO.load(fis);
                fis.close();
                return r;
            }
            else
            {
                file.createNewFile();
                return new HashRelation<String, String>();
            }
        }
        catch (Exception e)
        {
            e.printStackTrace();
            return null;
        }
    }


    static public void savePreferencesBelongingTo(Object object, Relation<String, String> r)
    {
        try
        {
            File file = getFile(object);
            FileOutputStream fos = new FileOutputStream(file);
            RelationIO.save(r, fos);
            fos.close();
//            System.out.println(file.getAbsolutePath()  + " saved");
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
    }

    private static File getFile(Object object)
    {
        return getFile(getID(object));
    }

    
    private static File getFile(int id)
    {
        File prefDir = new File(System.getProperty("user.home") + File.separator + ".lucci-flat-preferences/");
        
        if (!prefDir.exists())
        {
            prefDir.mkdirs();
        }

        return new File(prefDir.getAbsolutePath() + File.separator + id + ".relation");
    }

    private static int getID(Object object)
    {
        return Math.abs((object.getClass().getName() + object.hashCode()).hashCode());
    }

}
