package toools.gui;

import java.awt.Color;
import java.awt.Component;
import java.awt.GridLayout;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.Random;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JTextArea;
import javax.swing.text.BadLocationException;

import toools.util.assertion.Assertions;



/**
 * @author luc.hogie Created on Jun 18, 2004
 */
public class Utilities
{
    public static Color getRandomColor(Random random)
    {
        return getRandomColor(false, random);
    }

    public static String toRGBHexa(Color c)
    {
    	return Integer.toHexString(c.getRGB()).substring(2);
    }
    
    public static Color getColor(Color a, Color b, double r)
    {
        Assertions.ensure(0 <= r && r <= 1, "must be between 0 and 1");
        return new Color((int) (a.getRed() * (1 - r) + b.getRed() * r),
                (int) (a.getGreen() * (1 - r) + b.getGreen() * r),
                (int) (a.getBlue() * (1 - r) + b.getBlue() * r));
    }

    
    public static Color getRandomColor(boolean alpha, Random random)
    {
        int r = (int) toools.math.MathsUtilities.pickRandomBetween(0, 255, random);
        int g = (int) toools.math.MathsUtilities.pickRandomBetween(0, 255, random);
        int b = (int) toools.math.MathsUtilities.pickRandomBetween(0, 255, random);
        int a = alpha ? 255 : (int) toools.math.MathsUtilities.pickRandomBetween(0, 255, random);
        return new Color(r, g, b, a);
    }

    public static Color setAlpha(Color color, int alpha)
    {
        return new Color(color.getRed(), color.getGreen(), color.getBlue(), alpha);
    }

    public static String getCurrentLine(JTextArea ta)
    {
        try
        {
            int lineNumber = ta.getLineOfOffset(ta.getCaretPosition());
            int a = ta.getLineStartOffset(lineNumber);
            int b = ta.getLineEndOffset(lineNumber);
            String line = ta.getText(a, b - a - 1);
            return line;
        }
        catch (BadLocationException e)
        {
            // TODO Auto-generated catch block
            e.printStackTrace();
            return null;
        }

    }
    
	public static JFrame displayInJFrame(Component c, String title)
	{
		final JFrame f = new JFrame( title);

		f.addWindowListener(new WindowAdapter() {

			public void windowClosing(WindowEvent e)
			{
				f.dispose();
			}
		});

		JPanel p = new JPanel();
		p.setLayout(new GridLayout(1, 1));
		p.add(c);
		f.setContentPane(p);
		f.setSize(600, 450);
		f.setVisible(true);
		f.doLayout();
		f.repaint();
		f.pack();
		return f;
	}
}
