package toools.io;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Map;
import java.util.Set;

import sun.reflect.generics.reflectiveObjects.NotImplementedException;
import toools.ExceptionUtilities;
import toools.io.file.AbstractFile;
import toools.io.file.Directory;
import toools.io.file.RegularFile;

public class OnDiskMap<K, V> implements Map<K, V>
{

	private final Directory location;

	
	public OnDiskMap(Directory location)
	{
		if (location == null)
			throw new NullPointerException();

		this.location = location;

		if (!location.exists())
		{
			location.mkdirs();
		}
	}
	
	@Override
	public void clear()
	{
		for (AbstractFile f : location.getChildFiles())
		{
			f.delete();
		}
	}

	@Override
	public boolean containsKey(Object arg0)
	{
		return location.getChild(arg0.toString(), RegularFile.class).exists();
	}

	@Override
	public boolean containsValue(Object value)
	{
		throw new NotImplementedException();
	}

	@Override
	public Set<java.util.Map.Entry<K, V>> entrySet()
	{
		throw new NotImplementedException();
	}

	@Override
	public V get(Object key)
	{
		RegularFile f = getFileForObject(key);

		if (f == null || !f.exists())
		{
			return null;
		}
		else
		{
			try
			{
				return (V) Utilities.bytesToObject(Object.class, f.getContent());
			}
			catch (IOException e)
			{
				throw new IllegalStateException(ExceptionUtilities.toString(e));
			}
		}
	}
	

	public static void main(String[] args) throws IOException
	{
		OnDiskMap<Object, Object> db = new OnDiskMap<Object, Object>(new Directory(System.getProperty("user.home") + "/coucou"));
		db.put("coucou", new ArrayList());
		System.out.println(db.get("coucou"));
		db.delete();
	}

	public void delete()
	{
		getLocation().delete();
	}

	private RegularFile getFileForObject(Object id)
	{
		return new RegularFile(getLocation().getPath() + '/' + id.toString());
	}
	
	public Directory getLocation()
	{
		return location;
	}

	@Override
	public boolean isEmpty()
	{
		return getLocation().getChildFiles().isEmpty();
	}

	@Override
	public Set<K> keySet()
	{
		throw new NotImplementedException();
	}

	@Override
	public V put(K key, V value)
	{
		try
		{
			getFileForObject(key).setContent(Utilities.objectToBytes(value));
			return value;
		}
		catch (IOException e)
		{
			throw new IllegalStateException(ExceptionUtilities.toString(e));
		}
	}

	@Override
	public void putAll(Map<? extends K, ? extends V> m)
	{
		
	}

	@Override
	public V remove(Object key)
	{
		V v = get(key);
		getFileForObject(key).delete();
		return v;
	}

	@Override
	public int size()
	{
		return getLocation().getChildFiles().size();
	}

	@Override
	public Collection<V> values()
	{
		throw new NotImplementedException();
	}


}
