package toools.math;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import toools.text.TextUtilities;


public abstract class Matrix<X, Y, V>
{
	private String xLabel = "x";
	private String yLabel = "y";

	public String getXLabel()
	{
		return xLabel;
	}

	public void setXLabel(String label)
	{
		xLabel = label;
	}

	public String getYLabel()
	{
		return yLabel;
	}

	public void setYLabel(String label)
	{
		yLabel = label;
	}

	public abstract Collection<X> getXs();

	public abstract Collection<Y> getYs();

	public abstract void set(X x, Y y, V value);

	public abstract V get(X a, Y b);

	int pad = 4;

	@Override
	public String toString()
	{
		StringBuilder b = new StringBuilder();
		int ynameWidth = getLongestItemIn(getYs()) + pad;
		Map<X, Integer> colWidth = getColumsWidth();

		b.append(TextUtilities.repeat(' ', ynameWidth));

		for (X x : getXs())
		{
			String s = x.toString();
			b.append(TextUtilities.repeat(' ', colWidth.get(x) - s.length()) + s);
		}

		b.append('\n');
		b.append(TextUtilities.repeat('-', b.length()));
		b.append('\n');

		for (Y y : getYs())
		{
			String s = y.toString();
			b.append(s + TextUtilities.repeat(' ', ynameWidth - s.length()));

			for (X x : getXs())
			{
				V v = get(x, y);
				int w = colWidth.get(x);
				String xs = v.toString();
				b.append(TextUtilities.repeat(' ', w - xs.length()) + xs);
			}
			
			b.append('\n');

		}

		return b.toString();
	}

	private Map<X, Integer> getColumsWidth()
	{
		Map<X, Integer> map = new HashMap();

		for (X x : getXs())
		{
			map.put(x, Math.max(getLongestItemIn(getColumn(x)), x.toString().length()) + pad);
		}

		return map;
	}

	private List<V> getColumn(X x)
	{
		List<V> c = new ArrayList();

		for (Y y : getYs())
		{
			c.add(get(x, y));
		}

		return c;
	}

	private int getLongestItemIn(Collection c)
	{
		int m = 0;

		for (Object o : c)
		{
			String s = o.toString();

			if (s.length() > m)
			{
				m = s.length();
			}
		}

		return m;
	}

	public String toLaTeX()
	{
		List<X> elements = new ArrayList<X>(getXs());
		String s = "\\begin{tabular}{|" + TextUtilities.repeat("r", elements.size()) + "|}";

		for (X a : elements)
		{
			s += "\t";

			for (Y b : getYs())
			{
				V c = get(a, b);
				s += c + " & ";
			}

			s += " \\\n";
		}

		s += "\\end{tabular}";
		return s;
	}
}
