package toools.net;

import java.net.InetAddress;
import java.rmi.AccessException;
import java.rmi.NotBoundException;
import java.rmi.Remote;
import java.rmi.RemoteException;
import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import toools.math.relation.HashRelation;
import toools.math.relation.Relation;


public class RMIUtilities
{
	public static Relation<InetAddress, String> getRegistriesContent(Collection<InetAddress> ips)
	{
		Relation<InetAddress, String> r = new HashRelation<InetAddress, String>();
		
		for (InetAddress ip : ips)
		{
			try
			{
				Registry registry = LocateRegistry.getRegistry(ip.getHostAddress());
				r.addAll(ip, Arrays.asList(registry.list()));
			}
			catch (AccessException e)
			{
				e.printStackTrace();
			}
			catch (RemoteException e)
			{
				e.printStackTrace();
			}
		}

		return r;
	}
	
	
	/*
	 * Assumes that the remove object was bound to its classname. E.g.   registry.bind("java.util.ArrayList", new ArrayList());
	 */
	public static <T extends Remote> Map<InetAddress, T> getRMIRemotesByClass(Class<T> clazz, Collection<InetAddress> ips)
	{
		return (Map<InetAddress, T>) getRMIRemotesByName(clazz.getName(), ips);
	}

	public static Map<InetAddress, Remote> getRMIRemotesByName(String name, Collection<InetAddress> ips)
	{
		if (System.getSecurityManager() == null)
		{
			System.setSecurityManager(new SecurityManager());
		}

		Map<InetAddress, Remote> remotes = new HashMap<InetAddress, Remote>();

		for (InetAddress ip : ips)
		{
			try
			{
				Registry registry = LocateRegistry.getRegistry(ip.getHostAddress());
				remotes.put(ip, registry.lookup(name));
			}
			catch (NotBoundException ex)
			{
				ex.printStackTrace();
			}
			catch (AccessException e)
			{
				e.printStackTrace();
			}
			catch (RemoteException e)
			{
				e.printStackTrace();
			}
		}

		return remotes;
	}

	public final static int DEFAULT_RMI_PORT = 1099;
	/*
	public static Collection<InetAddress> getRMIServersInLANs()
	{
		Collection<InetAddress> ips = new ArrayList<InetAddress>();

		for (InetAddress ip : NetUtilities.getIPAddresses().getInverseRelation().getKeys())
		{
			if (!ip.isLoopbackAddress() && ip.getAddress().length == 4)
			{
				ips.addAll(NetUtilities.getWorkingServersInLANOf(DEFAULT_RMI_PORT, ip, 1000));
			}
		}

		return ips;
	}*/
}
