package toools.text;

import java.io.IOException;
import java.io.InputStream;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.Random;
import java.util.Vector;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import toools.Clazz;

public class TextUtilities
{

    public static int parseFileSize(String s)
    {
	return parseFileSize(s, 1024);
    }

    public static int parseFileSize(String s, int multiplier)
    {
	char lastLetter = s.charAt(s.length() - 1);

	if (lastLetter == 'b')
	{
	    s = s.substring(0, s.length() - 1);
	    lastLetter = s.charAt(s.length() - 1);
	}

	if (Character.isLetter(lastLetter))
	{
	    char unit = Character.toLowerCase(lastLetter);
	    int base = Integer.valueOf(s.substring(0, s.length() - 1));

	    if (unit == 'k')
	    {
		return base * multiplier;
	    }
	    else if (unit == 'm')
	    {
		return base * multiplier * multiplier;
	    }
	    else if (unit == 'g')
	    {
		return base * multiplier * multiplier * multiplier;
	    }
	    else
	    {
		throw new IllegalArgumentException("unknown unit '" + lastLetter + "'");
	    }
	}
	else
	{
	    return Integer.valueOf(s);
	}
    }

    public static String toString(double i)
    {
	if (i == (int) i)
	{
	    return String.valueOf((int) i);
	}
	else
	{
	    return String.valueOf(i);
	}
    }

    public static String toString(Exception ex)
    {
	StringWriter w = new StringWriter();
	PrintWriter pw = new PrintWriter(w);
	ex.printStackTrace(pw);
	pw.flush();
	return w.getBuffer().toString();
    }

    public static int compareLexicographically(InputStream as, InputStream bs) throws IOException
    {
	while (true)
	{
	    int ab = as.read();
	    int bb = bs.read();

	    if (ab == -1 && bb == -1)
	    {
		as.close();
		bs.close();
		return 0;
	    }
	    else if (ab == -1)
	    {
		as.close();
		bs.close();
		// a is shorter than b
		return -1;
	    }
	    else if (bb == -1)
	    {
		as.close();
		bs.close();
		// a is longer than b
		return 1;
	    }
	    else
	    {
		if (ab != bb)
		{
		    as.close();
		    bs.close();
		    return new Integer(ab).compareTo(bb);
		}
	    }
	}
    }

    public static String min(String a, String b)
    {
	return a.compareTo(b) > 0 ? b : a;
    }

    public static String max(String a, String b)
    {
	return a.compareTo(b) > 0 ? a : b;
    }

    public static List<String> splitInLines(String text)
    {
	return Arrays.asList(text.split("\\n"));
    }

    public static List<String> splitInLines2(String text)
    {
	List<String> l = new ArrayList();
	int p = 0;

	while (true)
	{
	    int q = text.indexOf('\n', p);

	    if (q < 0)
	    {
		l.add(text.substring(p));
		break;
	    }
	    else
	    {
		l.add(text.substring(p, q));
		p = q + 1;
	    }
	}

	return l;
    }

    public static List<Integer> grep(List<String> lines, String pattern, boolean caseSensitive, boolean v)
    {
	List<Integer> newLines = new ArrayList<Integer>();

	Pattern re = caseSensitive ? Pattern.compile(pattern) : Pattern.compile(pattern, Pattern.CASE_INSENSITIVE);

	for (int i = 0; i < lines.size(); ++i)
	{
	    String line = lines.get(i);

	    if ((!v && re.matcher(line).matches()) || (v && !re.matcher(line).matches()))
	    {
		newLines.add(i);
	    }
	}

	return newLines;
    }

    public static int indexOfRegexp(String lookIn, Pattern lookFor)
    {
	Matcher m = lookFor.matcher(lookIn);

	if (m.find())
	{
	    return m.start();
	}
	else
	{
	    return -1;
	}
    }

    public static List<String> getLinesAtIndexes(List<String> lines, List<Integer> lineNumbers, int indexShift)
    {
	List<String> selectedLines = new ArrayList<String>();

	for (int thisLineNumber : lineNumbers)
	{
	    selectedLines.add(lines.get(thisLineNumber + indexShift));
	}

	return selectedLines;
    }

    public static String prefixEachLineBy(String text, String prefix)
    {
	if (!text.startsWith("\n"))
	{
	    text = prefix + text;
	}

	return text.replaceAll("\n", "\n" + prefix);
    }

    public static void prefixEachLineBy(Collection<String> lines, String prefix)
    {
	Collection<String> newlines = (Collection<String>) Clazz.makeInstance(lines.getClass());

	for (String line : lines)
	{
	    newlines.add(prefix + line);
	}
    }

    public static void prefixEachLineByLineNumber(List<String> lines, String separator, int start)
    {
	int width = (int) Math.log10(lines.size()) + 1;

	for (int i = 0; i < lines.size(); ++i)
	{
	    lines.set(i, flushRight(String.valueOf(i + start), width, '0') + separator + lines.get(i));
	}
    }

    public static String toBinary(int n, int numberOfDigits, boolean useSpacing)
    {
	StringBuffer buf = new StringBuffer();

	while (numberOfDigits-- > 0)
	{
	    if (n % 2 == 0)
	    {
		buf.append('0');
	    }
	    else
	    {
		buf.append('1');
	    }

	    n /= 2;

	    if (useSpacing)
	    {
		if (numberOfDigits % 4 == 0)
		{
		    buf.append(' ');
		}
	    }
	}

	buf.reverse();
	return buf.toString();
    }

    public static String toHex(byte[] bytes)
    {
	return toHex(bytes, "");
    }

    public static String toHex(byte[] bytes, String sep)
    {
	StringBuilder buf = new StringBuilder();

	for (int i = 0; i < bytes.length; ++i)
	{
	    byte b = bytes[i];

	    if (i > 0)
	    {
		buf.append(sep);
	    }

	    String s = toHexString(b);

	    if (s.length() == 1)
	    {
		s = "0" + s;
	    }
	    
	    buf.append(s);
	}

	return buf.toString();
    }

    public static byte[] fromHex(String s)
    {
	if (s.length() % 2 != 0)
	    throw new IllegalArgumentException("input text doesn't have an even number of characters");

	byte[] r = new byte[s.length()];

	for (int i = 0; i < r.length; i += 2)
	{
	    String a = s.substring(i, i + 2);
	    r[i] = Byte.decode("#" + a);
	}

	return r;
    }

    public static String toHexString(byte b)
    {
	return Integer.toHexString(b & 0xFF);
    }

    public static boolean isDouble(String s)
    {
	try
	{
	    Double.valueOf(s);
	    return true;
	}
	catch (NumberFormatException ex)
	{
	    return false;
	}
    }

    public static boolean isInt(String s)
    {
	try
	{
	    Integer.valueOf(s);
	    return true;
	}
	catch (NumberFormatException ex)
	{
	    return false;
	}
    }

    /**
     * @return a nice representation of the name of the given array class.
     *         org.lucci.Boh -> org.lucci.Boh [String9 - String[]
     */
    public static String getNiceClassName(Class clazz)
    {
	if (clazz.isArray())
	{
	    return getNiceClassName(clazz.getComponentType()) + "[]";
	}
	else
	{
	    return clazz.getName();
	}
    }

    public static String getClassNameWithoutPackage(Class clazz)
    {
	if (clazz.isArray())
	{
	    return getNiceClassName(clazz.getComponentType()) + "[]";
	}
	else
	{
	    String s = clazz.getName();
	    return s.substring(s.lastIndexOf('.') + 1);
	}
    }

    public static String replaceVariableValues(String s, Map<String, String> variableValues)
    {
	int a = s.indexOf("${");

	if (a < 0)
	{
	    return s;
	}
	else
	{
	    int b = s.indexOf('}', a + 2);

	    if (b < 0)
	    {
		throw new IllegalArgumentException("unterminated variable reference: '}' expected");
	    }
	    else
	    {
		String name = s.substring(a + 2, b);

		if (name.matches("[a-zA-Z_]+"))
		{
		    String value = variableValues.get(name);

		    if (value == null)
		    {
			throw new IllegalArgumentException("variable undeclared: " + name);
		    }
		    else
		    {
			return s.substring(0, a) + value + replaceVariableValues(s.substring(b + 1), variableValues);
		    }
		}
		else
		{
		    throw new IllegalArgumentException("invalid variable name: '" + name + "'");
		}
	    }
	}
    }

    /**
     * @return the full name of the method described by the given elements.
     */
    public static String getNiceMethodName(Class target, String name, Class[] argTypes)
    {
	StringBuilder buf = new StringBuilder();
	buf.append(target.getName());
	buf.append('#');
	buf.append(name);
	buf.append('(');

	if (argTypes != null && argTypes.length > 0)
	{
	    for (int i = 0; i < argTypes.length; ++i)
	    {
		buf.append(' ');
		buf.append(getNiceClassName(argTypes[i]));

		if (i < argTypes.length - 1)
		{
		    buf.append(',');
		}
	    }

	    buf.append(' ');
	}

	buf.append(')');
	return buf.toString();
    }

    public static String normalizePropertyName(String s)
    {
	StringBuffer buf = new StringBuffer();
	int len = s.length();

	for (int i = 0; i < len; ++i)
	{
	    char c = s.charAt(i);

	    if (isPropertyChar(c))
	    {
		if (i == 0)
		{
		    buf.append(Character.toLowerCase(c));
		}
		else if (i > 0 && !isPropertyChar(s.charAt(i - 1)))
		{
		    buf.append(Character.toUpperCase(c));
		}
		else
		{
		    buf.append(c);
		}
	    }
	}

	return buf.toString();
    }

    public static boolean isPropertyChar(char c)
    {
	return Character.isLetterOrDigit(c) || c == '_';
    }

    public static String capitalizeWord(String s)
    {
	if (s.isEmpty())
	{
	    return "";
	}
	else
	{
	    return Character.toUpperCase(s.charAt(0)) + s.substring(1).toLowerCase();
	}
    }

    public static final Collection<String> exceptions = Arrays.asList(new String[] { "a", "the", "an", "and", "to",
	    "from", "on", "of", "by" });

    public static String capitalizeAllWords(String s)
    {
	List<String> words = Arrays.asList(s.split(" "));

	if (!words.isEmpty())
	{
	    words.set(0, capitalizeWord(words.get(0)));

	    for (int i = 1; i < words.size(); ++i)
	    {
		String w = words.get(i).toLowerCase();
		words.set(i, exceptions.contains(w) ? w : capitalizeWord(w));
	    }
	}

	return concatene(words, " ");
    }

    public static void main(String... args)
    {
	System.out.println(splitInLines2("cocou\n ca va ?\ndfsz"));
    }

    public static String invertCase(String s)
    {
	StringBuffer buf = new StringBuffer();
	int len = s.length();

	for (int i = 0; i < len; ++i)
	{
	    char c = s.charAt(i);

	    if (Character.isLetter(c))
	    {
		if (Character.isUpperCase(c))
		{
		    buf.append(Character.toLowerCase(c));
		}
		else
		{
		    buf.append(Character.toUpperCase(c));
		}
	    }
	    else
	    {
		buf.append(c);
	    }
	}

	return buf.toString();
    }

    public static String repeat(String s, int count)
    {
	StringBuilder buf = new StringBuilder();

	for (int i = 0; i < count; ++i)
	{
	    buf.append(s);
	}

	return buf.toString();
    }

    public static String repeat(char c, int count)
    {
	StringBuilder buf = new StringBuilder();

	for (int i = 0; i < count; ++i)
	{
	    buf.append(c);
	}

	return buf.toString();
    }

    public static String flushRight(String s, int lineLenght, char fillChar)
    {
	if (s.length() > lineLenght)
	    throw new IllegalArgumentException("length is smaller that original line length");

	return TextUtilities.repeat(String.valueOf(fillChar), lineLenght - s.length()) + s;
    }

    public static String flushLeft(String s, int lineLenght, char fillChar)
    {
	if (s.length() > lineLenght)
	    throw new IllegalArgumentException("string is larger than line length (" + lineLenght + "): " + s);

	return s + TextUtilities.repeat(String.valueOf(fillChar), lineLenght - s.length());
    }

    public static String concatene(List<String> strings, String separator)
    {
	StringBuilder b = new StringBuilder();
	int sz = strings.size();

	for (int i = 0; i < sz; ++i)
	{
	    b.append(strings.get(i));

	    if (i < sz - 1)
	    {
		b.append(separator);
	    }
	}

	return b.toString();
    }

    public static List<String> wrap(String text, int size)
    {
	List<String> unwrappables = Arrays.asList(text.split("\\s+"));

	for (int i = 0; i < unwrappables.size(); ++i)
	{
	}

	List<String> lines = new Vector<String>();
	lines.add("");

	for (String unwrappable : unwrappables)
	{
	    if (!lines.get(lines.size() - 1).isEmpty())
	    {
		unwrappable = ' ' + unwrappable;
	    }

	    if (lines.get(lines.size() - 1).length() + unwrappable.length() <= size)
	    {
		lines.set(lines.size() - 1, lines.get(lines.size() - 1) + unwrappable);
	    }
	    else
	    {
		lines.add(unwrappable.trim());
	    }
	}

	return lines;
    }

    public static char pickUpOneRandomChar(String s, Random random)
    {
	return s.charAt((int) (random.nextDouble() * s.length()));
    }

    public static String generateRandomString(String alphabet, int length, Random random)
    {
	StringBuffer b = new StringBuffer();

	while (length-- >= 0)
	{
	    b.append(pickUpOneRandomChar(alphabet, random));
	}

	return b.toString();
    }

    public static int computeLevenshteinDistance(String s, String t)
    {
	if (s == null || t == null)
	{
	    throw new NullPointerException();
	}

	/*
	 * The difference between this impl. and the previous is that, rather
	 * than creating and retaining a matrix of size s.length()+1 by
	 * t.length()+1, we maintain two single-dimensional arrays of length
	 * s.length()+1. The first, d, is the 'current working' distance array
	 * that maintains the newest distance cost counts as we iterate through
	 * the characters of String s. Each time we increment the index of
	 * String t we are comparing, d is copied to p, the second int[]. Doing
	 * so allows us to retain the previous cost counts as required by the
	 * algorithm (taking the minimum of the cost count to the left, up one,
	 * and diagonally up and to the left of the current cost count being
	 * calculated). (Note that the arrays aren't really copied anymore, just
	 * switched...this is clearly much better than cloning an array or doing
	 * a System.arraycopy() each time through the outer loop.)
	 * 
	 * Effectively, the difference between the two implementations is this
	 * one does not cause an out of memory condition when calculating the LD
	 * over two very large strings.
	 */

	int n = s.length(); // length of s
	int m = t.length(); // length of t

	if (n == 0)
	{
	    return m;
	}
	else if (m == 0)
	{
	    return n;
	}

	int p[] = new int[n + 1]; // 'previous' cost array, horizontally
	int d[] = new int[n + 1]; // cost array, horizontally
	int _d[]; // placeholder to assist in swapping p and d

	// indexes into strings s and t
	int i; // iterates through s
	int j; // iterates through t

	char t_j; // jth character of t

	int cost; // cost

	for (i = 0; i <= n; i++)
	{
	    p[i] = i;
	}

	for (j = 1; j <= m; j++)
	{
	    t_j = t.charAt(j - 1);
	    d[0] = j;

	    for (i = 1; i <= n; i++)
	    {
		cost = s.charAt(i - 1) == t_j ? 0 : 1;
		// minimum of cell to the left+1, to the top+1, diagonally left
		// and up +cost
		d[i] = Math.min(Math.min(d[i - 1] + 1, p[i] + 1), p[i - 1] + cost);
	    }

	    // copy current distance counts to 'previous row' distance counts
	    _d = p;
	    p = d;
	    d = _d;
	}

	// our last action in the above loop was to switch d and p, so p now
	// actually has the most recent cost counts
	return p[n];
    }

    public static String removeComments(String s)
    {
	return s.replaceAll("#.*(\\n|$)", "");
    }

    public static int getNumberOf(char c, String s)
    {
	int n = 0;
	int p = -1;
	
	while ((p = s.indexOf(c, p + 1)) != -1 )
	{
	    ++n;
	}
	
	return n;
    }

}