package toools.text.xml;

import java.io.ByteArrayInputStream;
import java.io.IOException;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

public class XMLUtilities
{
    public static Document xml2dom(String xml, boolean validating) throws SAXException

    {
	DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
	dbf.setValidating(validating);

	try
	{
	    return dbf.newDocumentBuilder().parse(new ByteArrayInputStream(xml.getBytes("UTF-8")));
	}
	catch (ParserConfigurationException e)
	{
	    e.printStackTrace();
	    throw new IllegalStateException();
	}
	catch (IOException e)
	{
	    e.printStackTrace();
	    throw new IllegalStateException();
	}
    }

    public static <V extends XMLNode> XMLNode dom2node(Node node)
    {
	XMLNode n = new XMLNode(node.getNodeName());
	NamedNodeMap attributes = node.getAttributes();

	if (attributes != null)
	{
	    for (int i = 0; i < attributes.getLength(); ++i)
	    {
		Node a = attributes.item(i);
		String name = a.getNodeName();
		String value = a.getTextContent();
		n.getAttributes().put(name, value);
	    }
	}

	NodeList domChildren = node.getChildNodes();

	for (int i = 0; i < domChildren.getLength(); ++i)
	{
	    Node childDomNode = domChildren.item(i);

	    if (childDomNode.getNodeName().equals("#text"))
	    {
		String content = childDomNode.getTextContent().trim();

		if (!content.isEmpty())
		{
		    XMLNode textNode = new TextNode(content);
		    textNode.setParent(n);
		}
	    }
	    else
	    {
		XMLNode child = dom2node(childDomNode);
		child.setParent(n);
	    }
	}

	return n;
    }

    public static XMLNode xml2node(String xml, boolean validating) throws SAXException
    {
	return dom2node(xml2dom(xml, validating).getDocumentElement());
    }

}
