package toools.thread;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

public abstract class CollectionMultiThreadProcessing
{
    private Object[] results;

    public CollectionMultiThreadProcessing(Collection<?> input)
    {
	this(input, Threads.getNumberOfThreadsANewTaskCanUseToTakeAllComputationalResources());
    }

    private int finishedThreads = 0;
    private boolean waiting = false;
    private Object lock = new Object();

    public CollectionMultiThreadProcessing(final Collection<?> input, final int numberOfThreads)
    {
	if (numberOfThreads < 0)
	    throw new IllegalArgumentException("negative number of threads");

	if (numberOfThreads > input.size())
	    throw new IllegalArgumentException("more threads than input objects");

	if (input.size() < 0)
	    throw new IllegalArgumentException("negative number of elements");

	this.results = new Object[numberOfThreads];

	final List<Thread> threads = new ArrayList<Thread>();
	final double div = input.size() / (double) numberOfThreads;

	for (int i = 0; i < numberOfThreads; ++i)
	{
	    final int threadID = i;
	    final int lb = (int) (div * i);
	    final int ub = (int) (div * (i + 1));

	    threads.add(new Thread() {
		public void run()
		{
		    results[threadID] = CollectionMultiThreadProcessing.this.run(threads, threadID, lb, ub);

		    synchronized (lock)
		    {
			if (++finishedThreads == numberOfThreads && waiting)
			{
			    lock.notify();
			}
		    }
		}
	    });
	}

	// now starts all threads
	for (Thread thisThread : threads)
	{
	    thisThread.start();
	}

	synchronized (lock)
	{
	    if (finishedThreads < numberOfThreads)
	    {
		try
		{
		    // System.out.println("waiting");
		    waiting = true;
		    lock.wait();
		}
		catch (InterruptedException e)
		{
		    e.printStackTrace();
		}
	    }
	}
    }

    protected abstract Object run(List<Thread> threads, int threadIndex, int lowerBound, int upperBound);

    public static void main(String[] args)
    {
	final ArrayList<String> input = new ArrayList();
	input.add("coucou");
	input.add("euh");
	input.add("alors voila");
	input.add("hehe");
	input.add("salut");
	System.out.println(input);

	new CollectionMultiThreadProcessing(input, 6) {

	    @Override
	    protected Object run(List<Thread> threads, int threadIndex, int lowerBound, int upperBound)
	    {
		System.out.println(threadIndex + " = " + lowerBound + " > " + upperBound);

		for (int i = lowerBound; i < upperBound; ++i)
		{
		    input.set(i, "length= " + input.get(i).length() + " (computed by thread " + threadIndex + ")");
		}

		return null;
	    }
	};

	System.out.println(input);

    }
}
