package toools.thread;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

public abstract class IndependantObjectMultiThreadProcessing<T>
{
	private final int numberOfThreads;
	private final List list;
	private int finishedThreads = 0;
	private boolean waiting = false;

	public IndependantObjectMultiThreadProcessing(Collection<T> input)
	{
		this(input, Runtime.getRuntime().availableProcessors() * 4);
	}

	public IndependantObjectMultiThreadProcessing(Collection<?> input, int numberOfThreads)
	{
		if (numberOfThreads < 1)
			throw new IllegalArgumentException("at least one thread is required to perform the task");

		if (input.size() < 1)
			throw new IllegalArgumentException("numberOfElements < 1");

		this.numberOfThreads = numberOfThreads;
		this.list = new ArrayList(input);
		execute();
	}

	int index = 0;
	Object lock = new Object();

	private void execute()
	{
		final List<Thread> threads = new ArrayList<Thread>();

		for (int i = 0; i < numberOfThreads; ++i)
		{
			threads.add(new Thread() {
				public void run()
				{
					while (true)
					{
						int i;
						synchronized (lock)
						{
							if (index >= list.size())
								break;
							i = index++;
						}

						IndependantObjectMultiThreadProcessing.this.run((T) list.get(i));
					}

					synchronized (lock)
					{
						if (++finishedThreads == numberOfThreads && waiting)
						{
							lock.notify();
						}
					}
				}
			});
		}

		// now starts all threads
		for (Thread thisThread : threads)
		{
			thisThread.start();
		}

		if (finishedThreads < numberOfThreads)
		{
			synchronized (lock)
			{
				try
				{
					waiting = true;
					lock.wait();
				}
				catch (InterruptedException e)
				{
					e.printStackTrace();
				}
			}
		}
	}

	protected abstract void run(T o);

}
