package toools.thread;

import java.util.ArrayList;
import java.util.List;

public abstract class MultiThreadProcessing
{
    public MultiThreadProcessing()
    {
	this(Runtime.getRuntime().availableProcessors());
    }

    private int finishedThreads = 0;
    private boolean waiting = false;
    private Object lock = new Object();

    public MultiThreadProcessing(final int numberOfThreads)
    {
	if (numberOfThreads < 0)
	    throw new IllegalArgumentException("negative number of threads");


	final List<Thread> threads = new ArrayList<Thread>();

	for (int i = 0; i < numberOfThreads; ++i)
	{
	    final int threadID = i;

	    threads.add(new Thread() {
		public void run()
		{
		    MultiThreadProcessing.this.run(threads, threadID);

		    synchronized (lock)
		    {
			if (++finishedThreads == numberOfThreads && waiting)
			{
			    lock.notify();
			}
		    }
		}
	    });
	}

	// now starts all threads
	for (Thread thisThread : threads)
	{
	    thisThread.start();
	}

	synchronized (lock)
	{
	    if (finishedThreads < numberOfThreads)
	    {
		try
		{
		    // System.out.println("waiting");
		    waiting = true;
		    lock.wait();
		}
		catch (InterruptedException e)
		{
		    e.printStackTrace();
		}
	    }
	}
    }

    protected abstract void run(List<Thread> threads, int threadIndex);

}
