package toools.thread;

import toools.extern.Proces;
import toools.math.MathsUtilities;

public class Threads
{
    public static long sleep(long ms)
    {
	// new IllegalStateException().printStackTrace();
	long a = System.currentTimeMillis();

	try
	{
	    Thread.sleep(ms);
	}
	catch (InterruptedException e)
	{
	    return System.currentTimeMillis() - a;
	}

	return ms;
    }

    public static void uninterruptibleSleep(long ms)
    {
	while (ms > 0)
	{
	    ms -= sleep(ms);
	}
    }

    public static void sleepForever()
    {
	uninterruptibleSleep(Long.MAX_VALUE);
    }

    public static void sleep(int i, String msg)
    {
	sleep(i);
	System.out.println(msg);
    }

    public static double getLastMinuteLoadAverage()
    {
	String uptimeOutput = new String(Proces.exec("uptime"));
	String[] values = uptimeOutput.split(" +");
	return Double.valueOf(values[values.length - 3]);
    }

    public static int getNumberOfProcessors()
    {
	return Runtime.getRuntime().availableProcessors();
    }

    public static double getNumberOfUnusedProcessors()
    {
	return getNumberOfProcessors() - getLastMinuteLoadAverage();
    }

    public static boolean isCPUOverloaded()
    {
	return getNumberOfUnusedProcessors() <= 0;
    }

    /**
     * If CPU usage is already 100%, there's no room for a new threads but it is
     * necessary to create at least one to perform the task.
     * 
     * @return
     */
    public static int getNumberOfThreadsANewTaskCanUseToTakeAllComputationalResources()
    {
	return Math.max(1, (int) MathsUtilities.round(getNumberOfUnusedProcessors(), 0));
    }

    public static void main(String[] args)
    {
	System.out.println(getNumberOfUnusedProcessors());
	System.out.println(getNumberOfThreadsANewTaskCanUseToTakeAllComputationalResources());
    }

}
