package toools.util.assertion;

import java.util.Arrays;
import java.util.LinkedList;
import java.util.List;

public class Assertions
{

    public static void ensureArgumentIsNotNull(Object o)
    {
        ensureArgument(o != null, "object was expected to be non-null");
    }

    public static void ensureArgument(boolean b, String msg)
    {
        if (!b)
        {
            throwException(new IllegalArgumentException(msg));
        }
    }

    
    public static void ensureThisCodeIsNeverExecuted()
    {
        ensure(false, "This code should never execute");
    }

    public static void ensure(boolean b)
    {
        ensure(b, "assertion failed!");
    }

    public static void ensure(boolean b, String msg)
    {
        if (!b)
        {
            throwException(new AssertionFailedException(msg));
        }
    }

    private static void throwException(RuntimeException e)
    {
        StackTraceElement[] trace = e.getStackTrace();
        List<StackTraceElement> list = new LinkedList<StackTraceElement>(Arrays.asList(trace));
        
        while (list.get(0).getClassName().endsWith("Assertions"))
        {
            list.remove(0);
        }

        e.setStackTrace(list.toArray(new StackTraceElement[0]));
        throw e;
    }
}
